#----------------------------------------------------------------------------
# Name:         ErrorDialogs.py
# Version:      1.0
# Created:      September--October 2001
# Author:       Chris Fama of Wholly Snakes Software,
#               Chris.Fama@whollysnakes.com
#----------------------------------------------------------------------------
"""
ErrorDialogs.py: by Christopher J. Fama (trading under the name
Wholly Snakes Software {Australian Business Number: 28379514278}).

This code is released under the auspices of the modified version of
the GNU Library Public License (LGPL) that constitutes the license
under which the GUI package wxPython is released [see the file
LICENSE.TXT accompanying that distribution).  I must also thank Graham
Boyd, of Boyd Mining, and CEO of the Minserve group of companies
(www.minserve.com.au), for kindly allowing the release of this
module under a "free" license, despite a certain part of it's
development having taken place while working for him...

Please note that this code, written for Python 2.1, is derives from
code written when Python 1.5.2 was current.  Although since Python 2.0
the sys.excepthook variable has been available, for ease and potential
backwards compatibility (or to be more realistic
backwards-PARTIAL-compatibility!), the code catches errors by
assigning a custom object to sys.stderr and monitoring calls to it's
writ) method.  Such calls which take place with a new value of
sys.last_traceback stand as evidence that at interpreter error has
just occurred; please note that this means that NO OTHER OUTPUT TO
sys.stderr WILL BE DISPLAYED.  THIS INCLUDES "warnings" generated by
the interpreter.  As far as I am aware, unless your code itself writes
to sys.stderr itself, these will be the only things you miss out
on--and many, if not most or all, of these will occur before your code
even gets the opportunity to set one of these file-like objects in
place.  If this is a problem for you and you can't work around it,
please contact means or the wxPython-users mailing list.


DOCUMENTATION:

This is a module to display errors--either explicitly requested by a
script or arbitrary interpreter errors--these needs arose in the
course of a commercial (contract) project, but were not developed in
the course of work on such [well, very little, albeit concurrently]...
[NBNB.Does not currently support display of other than interpreter errors.]

Usage, after 'from wxPython.lib.ErrorDialogs import *' (all
identifiers defined in this module begin with "wxPy", and many of them
with "wxPyError_", so there should be no namespace conflicts...):

    wxPyNewErrorDialog (<win> (['frame='] <frame (can be None)>,
                                         <OPTIONS>))
                   ...
    wxPyDestroyErrorDialogIfPresent () # e.g. when top frame destroyed)

for unhandled errors, or

    returnval = wxpyNonFatalError (<frame (can be None)>,
                          <HTML message>
                          [,<OPTIONS>])
or
    returnval = wxPyFatalError (<HTML message>,
                       [,<OPTIONS, an extra one of which may be
                          'frame=' <frame (defaults to None)>>])
or

    wxPybNonWindowingError (message)

for explicit errors.

<win> is one of
    wxPyNonFatalErrorDialog
    wxPyFatalErrorDialog
    wxPyFatalErrorDialogWithTraceback
    wxPyNonFatalErrorDialogWithTraceback
    wxPyNonWindowingErrorHandler

and the OPTIONS (with defaults) are: (please note that the options for
wxPyNonWindowingErrorHandler / wxPyNonWindowingError are 'almost' a (small))
subset of these):

    'modal' [default 1]: block until dismissed.

    'programname' [default "Python program"]: appears inThe
    caption of the dialog, amidst it's text and (by default) in mailings.

    'whendismissed' option, if given, this should be a string, to be
    executed in a restricted environment (see rexec module) after
    dialog dismissal.  Typically, this should be Python code to "clean
    up" after what was presumably the partial execution of some
    operation started by the user, after the unexpected interruption
    by some error [which--at least the way I work, means an unexpected
    error in the code, since exceptions that may be raised by some
    foreseen (in a programmatic sense) should normally be handled by
    try...except clauses].
    NOTE THAT CURRENTLY THE rexec CODE IS NOT WORKING, SO THIS IS JUST DONE
    BY exec...

    'mailto': if None, give no "e-mail support" option, otherwise this
    is meant to be an address for 'bug reports'; a command button will
    be created for this, which pops up another dialog  [Linux], or a window of
    another program [Windows].

    On Windows, this will launch your mailer (assuming your mailer
    would start when a file with the '.eml'extension is double-clicked
    in Windows Explorer--this will be the case for Microsoft Outlook
    Express [tm and all those legal necessities] if installed, and the
    association has not been taken over by some other program.  On
    Linux, it will open a file dialog to save the message, by default
    as a '.html' file...{Please note that, on Windows and with current
    (when I got the machine I'm writing this on--early 2000) versions
    of Outlook Express, you may need to enter your e-mail address in
    the box beside the "mail support" button.)

    The template for the mail message is in the 'MessageTemplate'
    attribute of the dialog-derived class in question (e.g.,
    wxPyNonFatalErrorDialog).  Search for this in the code below to
    see the default (note that this template is in HTML format).  This
    attributes uses the '%{name}s' string conversion feature of Python
    [see sec.2 of the Python Library Reference]; allowed Lance are:
    programname version exceptionname exceptionvalue
    extraexceptioninformation traceback

    'version': str(this) will appear after 'Version' below "Error in
    <programname>" at the top of the dialog.

EXAMPLES:

    sys.stderr = wxPyNonFatalErrorWindowWithTraceback (
        parentframe,
        programname='sumthing',
        mailto='me@sumwear',
        whendismissed="from wxPython.wx import * ; wxBell()")

FOR INTERNATIONAL [NON-ENGLISH-SPEAKING] USE:
    wxPyNonFatalErrorDialog and relatives have the method
        SetText(Number NUMBER, STRING)

    where STRING is to displayed in the wxStaticText control with ID
    wxPyError_ID_TEXT<NUMBER>--see the automatically-generated code
    for information about the meaning of these...

"""

_debug = 0
#_debug = 1 # uncomment to display some information (to stdout)
Version = 1.3

from wxPython.wx import *
import string, sys, traceback, time, rexec, operator, types, cStringIO, os
#from wxPython.lib.createandsendHTMLmail import *# now inline
#import MimeWriter, mimetools, tempfile, smtplib
import urllib, webbrowser

from ErrorDialogs_wdr import *

# You may see from the above line that I used the excellent RAD tool
# wxDesigner, by Robert Roebling, to accelerate development of this
# module...  The above is left for the convenience of making future
# changes with wxDesigner; also so the wxDesigner-generated codedoes
# not need to precede the "hand-generated" code in this file; finally,
# as a personal endorsement: it is truly a brilliant time-saver!
# Please note that, at the time of writing, the wxDesigner-generated
# output requires manual removal of the PythonBitmaps function--an
# appropriate version of this function will be imported from a
# similarly- named module.  Another manual change will have to be made
# to the automatically-generated source: "parent.sizerAroundText = "
# should be added [immediately] before the text similar to "item13 =
# wxStaticBoxSizer( item14, wxVERTICAL )", this sizer being the one
# containing the wxTextCtrl...  [IMPORTANT NOTE: THIS LINE SHOULD BE
# THE ONE INSTANTIATING A wxStat2icBoxSizer, *NOT* THE wxStaticBox
# ITSELF...]  As of version 1.2 [November 2001], this also needs to be
# done for the {sizers around the}   wxPyClickableHtmlWindow's generated in
# populate_wxPyNonFatalError and populate_wxPyFatalError--note that
# for ease this sizer is still called "sizerAroundText"...

def wxPyDestroyErrorDialogIfPresent():
    if isinstance(sys.stderr,wxPyNonFatalErrorDialog):
        sys.stderr.Destroy()
    sys.stderr = None

def wxPyNewErrorDialog(dlg):
    wxPyDestroyErrorDialogIfPresent()
    sys.stderr = dlg

class wxPyNonWindowingErrorHandler:
    this_exception = 0
    softspace = 0
    def __init__(self,fatal=0,file=sys.__stderr__):
        self.fatal = fatal
        self.file = file
    def write(self,s):
        import sys
        if s.find("Warning") <> 0\
           and  self.this_exception is not sys.last_traceback:
            wxPyNonWindowingError("The Python interpreter encountered an error "
                              "not handled by any\nexception handler--this "
                              "may represent some programming error.",
                              fatal=self.fatal,
                               stderr=self.file,
                              last=1)
        self.this_exception = sys.last_traceback

def wxPyNonWindowingError(msg,#output=None,errors=None,
                          stderr=sys.__stderr__,
                          fatal=1,
                          last=None):
    if os.path.exists("wxPyNonWindowingErrors.txt"):
        mode = 'a+'
    else:
        mode = 'w'
    fl = open("wxPyNonWindowingErrors.txt",mode)
    if stderr is not None:
        l = [fl,stderr] # so that the error will be written to the file
            # before any potential error in stderr.write()... (this is largely
            # for my own sake in developing...)
    else:
        l = [fl]
    for f in l:
        f.write(time.ctime (time.time ()) + ": ")
        f.write(msg)
        #f.flush()
        if sys.exc_info () [0] is not None:
            if last:
                f.write('Currently handled exception:\n')
                f.flush()
            traceback.print_exc(file=f)
            if last:
                f.write('\nPrevious (?) error:\n')
        elif last or sys.last_traceback:
            f.write("\n\n(For wizards only) ")
        if last:
            if type(last) <> types.ListType or len(last) < 3:
                if (hasattr(sys,"last_traceback") and sys.last_traceback):
                    last = [sys.last_type ,sys.last_value,sys.last_traceback]
            if type(last) == types.ListType:
                traceback.print_exception(last[0],last[1],last[2],
                                          None,f)
         #f.flush()
        if f is sys.__stderr__:
            s = ' (see the file "wxPyNonWindowingErrors.txt")'
        else:
            s = ""
        f.write("Please contact the author with a copy of this\n"
                "message%s.\n" % s)
        #f.flush()
    fl.close()
    if fatal and stderr is sys.__stderr__:
        if sys.__stderr__ and sys.platform in ["windows",'nt',"win32"]:
            sys.__stderr__.write(
                "\nYou may have to manually close this window to exit.")
        sys.exit()

class wxPythonRExec (rexec.RExec):
    def __init__(self,securityhole=0,*args,**kwargs):
        apply(rexec.RExec.__init__, (self,) + args, kwargs)
        if securityhole:
            self.ok_builtin_modules = self.ok_builtin_modules + \
            ('wxPython', 'wxPython.wxc','wxPython.wx','wxPython.misc',
             'wxPython.misc2', 'wxPython.windows', 'wxPython.gdi',
             'wxPython.clip_dnd', 'wxPython.events', 'wxPython.mdi',
             'wxPython.frames', 'wxPython.stattool', 'wxPython.controls',
             'wxPython.controls2', 'wxPython.windows2', 'wxPython.cmndlgs',
             'wxPython.windows3', 'wxPython.image', 'wxPython.printfw',
             'wxc','misc', 'misc2', 'windows', 'gdi', 'clip_dnd', 'events',
             'mdi', 'frames', 'stattool', 'controls', 'controls2', 'windows2',
             'cmndlgs', 'windows3', 'image', 'printfw', 'wx')
            # possible security hole!

##def wxPyFatalError(msg,frame=None,**kwargs):
##    kwargs.update({'fatal' : 1})
##    apply(wxPyNonFatalError,
##          (frame,msg),
##          kwargs)

class wxPyNonFatalErrorDialogWithTraceback(wxDialog):
    this_exception = 0
    populate_function = populate_wxPyNonFatalErrorDialogWithTraceback
    no_continue_button = False
    fatal = False
    modal = True
    exitjustreturns = False # really only for testing!

    def __init__(self, parent, id,
                 pos=wxPyDefaultPosition,
                 size=wxPyDefaultSize,
                 style=wxDEFAULT_DIALOG_STYLE,
                 programname="Python program",
                 version="?",
                 mailto=None,
                 whendismissed="",
                 extraversioninformation="",
                 caption="Python error!",
                 versionname=None,
                 errorname=None,
                 disable_exit_button=False):

        if self.fatal:
            whetherNF = ""
        else:
            whetherNF = "non-"
        title = "A (%sfatal) error has occurred in %s!"\
                % (whetherNF,programname)
        self.programname = programname # save for later use
        self.mailto = mailto # save for later use
        self.parent = parent # save for later use
        self.whendismissed = whendismissed # save for later use
        self.dialogtitle = title # save for later use

        wxDialog.__init__(self, parent, id, title, pos, size, style)

        self.topsizer = self.populate_function( False,True )

        self.SetProgramName(programname)
        self.SetVersion(version)
        if errorname:
            self.FindWindowById(wxPyError_ID_TEXT1).SetLabel(str(errorname))
        if versionname:
            self.FindWindowById(wxPyError_ID_TEXT2).SetLabel(str(versionname))
        self.FindWindowById(wxPyError_ID_VERSIONNUMBER).SetLabel(str(version))
        self.FindWindowById(wxPyError_ID_EXTRA_VERSION_INFORMATION).SetLabel(str(
            extraversioninformation))
        if caption:
            self.SetTitle(caption)

        if not self.no_continue_button:
            EVT_BUTTON(self, wxPyError_ID_CONTINUE, self.OnContinue)
        if mailto:
            disable_mail_button = 0
        else:
            disable_mail_button = 1
        if not disable_mail_button:
            EVT_BUTTON(self, wxPyError_ID_MAIL, self.OnMail)
        else:
            self.GetMailButton().Enable(False)
        # disable the entry box for an e-mail address by default (NOT PROPERLY DOCUMENTED)
        if not hasattr(self,"enable_mail_address_box"):
            self.FindWindowById(wxPyError_ID_ADDRESS).Enable(False)
        if not disable_exit_button:
            EVT_BUTTON(self, wxPyError_ID_EXIT, self.OnExit)

    def GetExtraInformation(self):
        return self.extraexceptioninformation

    def SetExtraInformation(self,value):
        self.extraexceptioninformation = value
        c = self.GetExtraInformationCtrl()
        if c is not None:
            c.SetLabel(str(value))
            self.topsizer.Layout()

    def GetExtraInformationCtrl(self):
        return self.FindWindowById(wxPyError_ID_EXTRAINFORMATION)

    def GetExceptionName(self):
        return str(self.exceptiontype)

    def SetExceptionName(self,value):
        self.exceptiontype = str(value)
        c = self.GetExceptionNameCtrl()
        if c is not None:
            c.SetLabel(str(value))
            self.topsizer.Layout()

    def GetExceptionNameCtrl(self):
        return self.FindWindowById(wxPyError_ID_EXCEPTIONNAME)

    def GetTraceback(self):
        try:
            return self.traceback
        except AttributeError:
            return None

    def SetTraceback(self,value):
        self.traceback = value
        c = self.GetTracebackCtrl()
        if c is not None:
            s,cs = c.GetSize(), c.GetClientSize()
            if value[-1] == '\n':
                value = value[:-1]
            if _debug:
                print "%s.SetTraceback(): ...SetValue('%s' (^M=\\r; ^J=\\n))"\
                      % (self,value.replace('\n',"^J"))
            c.SetValue(value)

            # Despite using the wxADJUST_MINSIZE flag in the
            # appropriate AddWindow method of the sizer, this doesn't
            # size the control appropriately... evidently the control's
            # GetBestSize method is not returning the "correct"
            # value...  So we perform a rather ugly "fix"... note that
            # this also requires that we remove the wxADJUST_MINSIZE
            # flag from the AddWindow method of the sizer containing
            # the wxTextCtrl, which adds the wxTextCtrl...  (this
            # amounts, as of wxDesigner 2.6, to only a few mouse
            # clicks...)

            if _debug:
                size = c.GetBestSize()
                print "%s.SetTraceback(): %s.GetBestSize() = (%s,%s)"\
                      % (self,c,size.width,size.height)
            w,h = 0,0
            for v in value.split("\n"):
                pw,ph,d,e = t = c.GetFullTextExtent(v)
                if _debug:
                    print v, t
                h = h + ph + e# + d
                pw = pw + wxSystemSettings_GetSystemMetric(wxSYS_VSCROLL_X)
                if pw > w:
                    w = pw
            w = w + s.width - cs.width
            h = h + s.height - cs.height
            if _debug:
                print "%s.SetTraceback(): calculated w,h =" % c,\
                      w,h,"and sys.platform = '%s'" % sys.platform
            self.sizerAroundText.SetItemMinSize (c,w,h)
            c.SetSize ((w,h))
            c.SetSizeHints (w,h,w,h)
            c.Refresh()#.SetAutoLayout(False)

            #^ the reason we need the above seems to be to replace the
            #faulty GetBestSize of wxTextCtrl...
            #self.sizerAroundText.Layout()
            self.topsizer.Layout()

    def GetTracebackCtrl(self):
        return self.FindWindowById(wxPyError_ID_TEXTCTRL)

    def GetVersion(self):
        return self.version

    def SetVersion(self,value):
        self.version = value
        c = self.GetVersionNumberCtrl()
        if c is not None:
            c.SetLabel(value)
            self.topsizer.Layout()

    def GetVersionNumberCtrl(self):
        return self.FindWindowById(wxPyError_ID_VERSIONNUMBER)

    def GetProgramName(self):
        return self.programname

    def SetProgramName(self,value):
        self.programname = value
        c = self.GetProgramNameCtrl()
        if c is not None:
            c.SetLabel(value)
            self.topsizer.Layout()

    def GetProgramNameCtrl(self):
        return self.FindWindowById(wxPyError_ID_PROGRAMNAME)

    def GetContinueButton(self):
        return self.FindWindowById(wxPyError_ID_CONTINUE)

    def GetMailButton(self):
        return self.FindWindowById(wxPyError_ID_MAIL)

    def GetExitButton(self):
        return self.FindWindowById(wxPyError_ID_EXIT)

    # write handler (which is really the guts of the thing...
    # [Note that this doesn't use sys.excepthook because I already had a
    # working body of code...

    def write(self,s):
        if self.this_exception is not sys.last_traceback:
          if not wxThread_IsMain():
             # Aquire the GUI mutex before making GUI calls.  Mutex is released
             # when locker is deleted at the end of this function.
             locker = wxMutexGuiLocker()

          self.this_exception = sys.last_traceback
            # this is meant to be done once per traceback's sys.stderr.write's
            # - on the first in fact.....
          try:
            #from wxPython.wx import wxBell
            wxBell()

            if _debug:
                if sys.stdout: sys.stdout.write(
                    'in  %s.write(): ' % self)

            self.exceptiontype = sys.last_type
            self.extraexceptioninformation = sys.last_value
            c = cStringIO.StringIO()
            traceback.print_last(None,c)
            self.traceback = c.getvalue()

            if _debug:
                #import traceback
                traceback.print_last(None,sys.stdout)

            self.SetExceptionName(str(self.exceptiontype))
            self.SetExtraInformation(str(self.extraexceptioninformation))
            self.SetTraceback(str(self.traceback))

            self.topsizer.Fit(self)
            self.topsizer.SetSizeHints(self)
            self.CentreOnScreen()

            if self.modal:
                self.ShowModal()
            else:
                self.Show(True)

          except:
              if not locals().has_key("c"):
                  c = cStringIO.StringIO()
                  c.write("[Exception occurred before data from "
                          "sys.last_traceback available]")
              wxPyNonWindowingError("Warning: "
                                "a %s error was encountered trying to "
                                "handle the exception\n%s\nThis was:"#%s\n"
                                % (sys.exc_type, c.getvalue()),#, c2.getvalue()),
                                stderr=sys.stdout,
                                last=0)


    # button handlers:

    def OnContinue(self, event):
     try:
        if self.whendismissed:
            parent = self.parent # so whendismissed can refer to "parent"
            if 1:
                if _debug:
                    if sys.stdout: sys.stdout.write("exec '''%s''': "
                                         % (self.whendismissed))
                exec self.whendismissed
                if _debug: print "\n",
            else:
                if _debug:
                    if sys.stdout: sys.stdout.write("wxPythonRExec(%s).r_exec('''%s'''): "
                                         % (self.securityhole,
                                            self.whendismissed))
                wxPythonRExec(self.securityhole).r_exec(self.whendismissed)
                if _debug: print "\n",
        if self.modal:
            self.EndModal(wxID_OK)
        else:
            self.Close ()
        if _debug: print "reimporting ",
        for  m in sys.modules.values():
            if m and m.__dict__["__name__"][0] in string.uppercase:#hack!
                if _debug:
                    print m.__dict__["__name__"],
                reload (m)
                if _debug:
                    print ' ',
        if _debug:
            print '\nENDING %s.OnContinue()..\n\n' % (self,),
     except:
      wxPyNonWindowingError("Warning: the following exception information"
                        " may not be the full story.. (because "
                        "a %s(%s) error was encountered trying to "
                        "handle the exception)\n\n"
                        % tuple(sys.exc_info()[:2]),
                        stderr=sys.stdout,
                        last=0)

    PlainMessageTemplate = \
                 "Hello,\n\n"\
                 '%(programname)s'\
                 " error.\n\n"\
                 "I encountered the following error when running your "\
                 'program %(programname)s,'\
                 "at %(date)s.\n\n"\
                 "(The following has been automatically generated...)\n"\
                 '%(traceback)s\n\n'\
                 "More information follows:\n\n"\
                 '[Insert more '\
                 "information about the error here, such as what you were "\
                 "trying to do at the time of the error.  Please "\
                 "understand that failure to fill in this field will be "\
                 "interpreted as an invitation to consign this e-mail "\
                 "straight to the trash can!]\n\n"\
                 'Yours sincerely,\n'\
                 "[insert your name here]\n"

    HTMLMessageTemplate = \
                 '<html>\n'\
                 '<body>'\
                 "<p>\n"\
                 "<i><b>Hello,</b></i>\n<p>\n"\
                 '<p><h2><font color="#CC6C00">%(programname)s</font>'\
                 " error.</h2>\n"\
                 "I encountered the following error when running your "\
                 'program <font color="#CC6C00">%(programname)s</font>,'\
                 "at %(date)s.\n<p>\n"\
                 "<p>"\
                 "<h2>Traceback (automatically generated):</h2>\n"\
                 '<p><font size="-1">\n<pre>%(traceback)s</pre>\n<p></font><p>'\
                 "\n<p>\n<h2>More information follows:</h2>\n<p>\n"\
                 '<font color="#CC6C00">'\
                 '<i>[Insert more '\
                 "information about the error here, such as what you were "\
                 "trying to do at the time of the error.  Please "\
                 "understand that failure to fill in this field will be "\
                 "interpreted as an invitation to consign this e-mail "\
                 "straight to the trash can!]\n</i><p>\n"\
                 "</font><p>\n"\
                 '<i><b>Yours sincerely,</b></i>\n<p>'\
                 '<font color="#CC6C00">'\
                 "[insert your name here]\n"\
                 "</font>\n"\
                 "</body>\n"\
                 "</html>\n"
    # text="#000000" bgcolor="#FFFFFF">\n'\

    def OnMail(self,event):
      try:
        if _debug:
            print 'Attempting to write mail message.\n',
        gmtdate = time.asctime(time.gmtime(time.time())) + ' GMT'
        tm = time.localtime(time.time())
        date = time.asctime(tm) + ' ' +\
               time.strftime("%Z",tm)
        programname = self.programname
        traceback = self.traceback
        mailto = self.mailto

        subject = "Un-caught exception when running %s." % programname
        mailfrom = None#self.FindWindowById (wxPyError_ID_ADDRESS)
        if mailfrom:
            mailfrom = mailfrom.GetValue()
        if _startmailerwithhtml(mailto,subject,
                                self.HTMLMessageTemplate % vars(),
                                text=self.PlainMessageTemplate % vars(),
                                mailfrom=mailfrom):
          if not (hasattr(self,"fatal") and self.fatal):
            self.OnContinue(event) # if ok, then act as if "Continue" selected
      except:
          wxPyNonWindowingError("Warning: the following exception information"
                                " may not be the full story... (because "
                                "a %s error was encountered trying to "
                                "handle the original exception)\n\n"#%s"
                                % (sys.exc_type,),#self.msg),
                                stderr=sys.stdout,
                                last=0)

    def OnExit(self, event):
        if self.IsModal():
            self.EndModal(wxID_CANCEL)
        if self.exitjustreturns:
            return
        wxGetApp().ExitMainLoop()

    def SetText(self,number,string):
        self.FindWindowById(eval("wxPyError_ID_TEXT%d"
                                 % number)).SetLabel(string)
        self.topsizer.Layout()

class wxPyFatalErrorDialogWithTraceback(wxPyNonFatalErrorDialogWithTraceback):
    populate_function = populate_wxPyFatalErrorDialogWithTraceback
    no_continue_button = True
    fatal = True

class wxPyNonFatalErrorDialog(wxPyNonFatalErrorDialogWithTraceback):
    populate_function = populate_wxPyNonFatalErrorDialog

class wxPyFatalErrorDialog(wxPyFatalErrorDialogWithTraceback):
    populate_function = populate_wxPyFatalErrorDialog

def _startmailerwithhtml(mailto,subject,html,text=None,mailfrom=None):
    if sys.hexversion >= 0x02000000:#\
        # and sys.platform in ["windows",'nt',"w is in32"]:
        s = 'mailto:%s?subject=%s&body=%s' % (mailto,
                                               urllib.quote(subject),
                                               urllib.quote(
            text.replace('\n','\r\n'),
            ""))

        # Note that RFC 2368 requires that line breaks in the body of
        # a message contained in a mailto URL MUST be encoded with
        # "%0D%0A"--even on Unix/Linux.  Also note that there appears
        # to be no way to specify that the body of a mailto tag be
        # interpreted as HTML (mailto tags shouldn't stuff around with
        # the MIME-Version/Content-Type headers, the RFC says, so I
        # can't even be bothered trying, as the Python
        # webbrowser/urllib modules quite likely won't allow
        # this... anyway, a plain text message is [at least!] fine...).

        if _debug:
            t = urllib.quote(text)
            if len(t) > 20 * 80:
                t = t[0:20 * 80] + "..."
                print "\nSummarizing (only shortened version of argument "\
                      "printed here), ",
            print 'webbrowser.open("' \
                  'mailto:%s?subject=%s&body=%s"' % (mailto,
                                                     urllib.quote(subject),
                                                     t)
        webbrowser.open(s)
        return 1
    else:
        return _writehtmlmessage(mailto,subject,html,text,mailfrom=mailfrom)

def _writehtmlmessage(mailto,subject,html,text=None,parent=None,mailfrom=None):
    dlg = wxFileDialog (parent,
                        "Please choose a a file to save the message to...",
                        ".",
                        "bug-report",
                        "HTML files (*.htm,*.html)|*.htm,*.html|"
                        "All files (*)|*",
                        wxSAVE | wxHIDE_READONLY)
    if dlg.ShowModal() <> wxID_CANCEL:
        f = open(dlg.GetPath(),"w")
        dlg.Destroy()
        f.write(_createhtmlmail(html,text,subject,to=mailto,mailfrom=mailfrom))
        f.close()
        return 1
    else:
        return 0

# PLEASE NOTE THAT THE CODE BELOW FOR WRITING A GIVEN
#(HTML) MESSAGE IS BY ART GILLESPIE [with slight modifications by yours truly].

def _createhtmlmail (html, text, subject, to=None, mailfrom=None):
    """Create a mime-message that will render HTML in popular
       MUAs, text in better ones (if indeed text is not unTrue (e.g. None)
       """
    import MimeWriter, mimetools, cStringIO

    out = cStringIO.StringIO() # output buffer for our message
    htmlin = cStringIO.StringIO(html)
    if text:
            txtin = cStringIO.StringIO(text)

    writer = MimeWriter.MimeWriter(out)
    #
    # set up some basic headers... we put subject here
    # because smtplib.sendmail expects it to be in the
    # message body
    #
    if mailfrom:
            writer.addheader("From", mailfrom)
            #writer.addheader("Reply-to", mailfrom)
    writer.addheader("Subject", subject)
    if to:
            writer.addheader("To", to)
    writer.addheader("MIME-Version", "1.0")
    #
    # start the multipart section of the message
    # multipart/alternative seems to work better
    # on some MUAs than multipart/mixed
    #
    writer.startmultipartbody("alternative")
    writer.flushheaders()
    #
    # the plain text section
    #
    if text:
        subpart = writer.nextpart()
        subpart.addheader("Content-Transfer-Encoding", "quoted-printable")
        pout = subpart.startbody("text/plain", [("charset", 'us-ascii')])
        mimetools.encode(txtin, pout, 'quoted-printable')
        txtin.close()
    #
    # start the html subpart of the message
    #
    subpart = writer.nextpart()
    subpart.addheader("Content-Transfer-Encoding", "quoted-printable")
    pout = subpart.startbody("text/html", [("charset", 'us-ascii')])
    mimetools.encode(htmlin, pout, 'quoted-printable')
    htmlin.close()
    #
    # Now that we're done, close our writer and
    # return the message body
    #
    writer.lastpart()
    msg = out.getvalue()
    out.close()
    return msg

def _sendmail(mailto,subject,html,text):# currently unused
    """For illustration only--this function is not actually used."""
    import smtplib
    message = _createhtmlmail(html, text, subject)
    server = smtplib.SMTP("localhost")
    server.sendmail(mailto, subject, message)
    server.quit()

def wxPyFatalError(parent,msg,**kw):
    return wxPyFatalOrNonFatalError(parent,msg,fatal=1,**kw)

def wxPyNonFatalError(parent,msg,**kw):
    return wxPyFatalOrNonFatalError(parent,msg,fatal=0,**kw)

def wxPyResizeHTMLWindowToDispelScrollbar(window,
                                         fraction,
                                         sizer=None,
                                         defaultfraction=0.7):
    # Try to `grow' parent window (typically a dialog), only so far as
    # no scrollbar is necessary, mantaining aspect ratio of display.
    # Will go no further than specified fraction of display size.
    w = 200
    if type(fraction) == type(''):
        fraction = int(fraction[:-1]) / 100.
    ds = wxDisplaySize ()
    c = window.GetInternalRepresentation ()
    while w < ds[0] * fraction:
        c.Layout(w)
        if _debug:
            print '(c.GetHeight() + 20, w * ds[1]/ds[0]):',\
                  (c.GetHeight() + 20, w * ds[1]/ds[0])
        if c.GetHeight() + 20 < w * ds[1]/ds[0]:
            size = (w,min(int ((w) * ds[1]/ds[0]),
                          c.GetHeight()))
            break
        w = w + 20
    else:
        if type(defaultfraction) == type(''):
            defaultfraction = int(defaultfraction[:-1]) / 100.
        defaultsize = (defaultfraction * ds[0], defaultfraction * ds[1])
        if _debug:
            print 'defaultsize =',defaultsize
        size = defaultsize
    window.SetSize(size)
    if sizer is not None:
        sizer.SetMinSize(size)
        sizer.Fit(window)
        #sizer.SetSizeHints(window)

def wxPyFatalOrNonFatalError(parent,
                             msg,
                             fatal=0,
                             extraversioninformation="",
                             caption=None,
                             versionname=None,
                             errorname=None,
                             version="?",
                             programname="Python program",
                             tback=None,# backwards compatibility, and for
                             #possible future inclusion of ability to display
                             #a traceback along with the given message
                             #"msg"... currently ignored though...
                             modal=0):
    if not wxThread_IsMain():
        # Aquire the GUI mutex before making GUI calls.  Mutex is released
        # when locker is deleted at the end of this function.
        locker = wxMutexGuiLocker()

    dlg = wxDialog(parent,-1,"Error!")

    if fatal:
        populate_function = populate_wxPyFatalError
    else:
        populate_function = populate_wxPyNonFatalError

    sizer = populate_function(dlg,False,True)

    window = dlg.FindWindowById(wxPyError_ID_HTML)
    window.SetPage(msg)
    wxPyResizeHTMLWindowToDispelScrollbar(window,
                                          "85%",
                                          sizer=dlg.sizerAroundText)
    dlg.FindWindowById(wxPyError_ID_PROGRAMNAME).SetLabel(str(programname))
    if errorname:
        dlg.FindWindowById(wxPyError_ID_TEXT1).SetLabel(str(errorname))
    if versionname:
        dlg.FindWindowById(wxPyError_ID_TEXT2).SetLabel(str(versionname))
    dlg.FindWindowById(wxPyError_ID_VERSIONNUMBER).SetLabel(str(version))
    dlg.FindWindowById(wxPyError_ID_EXTRA_VERSION_INFORMATION).SetLabel(str(
        extraversioninformation))
    if caption:
        dlg.SetTitle(caption)
    sizer.Fit(dlg)
    sizer.SetSizeHints(dlg)
    dlg.CentreOnScreen()

    if modal:
        v = dlg.ShowModal()
        dlg.Destroy()
        return v
    else:
        dlg.Show(True)




"""wxStyledTextControl constants for documentation.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: StyledTextConstants.py,v 1.1.2.1 2003/03/11 19:37:11 RD Exp $"
__revision__ = "$Revision: 1.1.2.1 $"[11:-2]


from wxPython import wx
from wxPython import stc


"""
>>> from wxPython import stc_c
>>> def gen_constants():
...     print '# Begin generated constants.'
...     for item in dir(stc_c):  # dir returns a sorted list.
...         if not callable(stc_c.__dict__[item]) \
...         and not item.startswith('__'):
...             print item, '=', 'stc.' + item, ' #', \
...                   repr(stc_c.__dict__[item])
...     print '# End generated constants.'

"""

# Begin generated constants.
STC_USE_DND = stc.STC_USE_DND  # 1
wxEVT_STC_CHANGE = stc.wxEVT_STC_CHANGE  # 10206
wxEVT_STC_CHARADDED = stc.wxEVT_STC_CHARADDED  # 10208
wxEVT_STC_DOUBLECLICK = stc.wxEVT_STC_DOUBLECLICK  # 10213
wxEVT_STC_DO_DROP = stc.wxEVT_STC_DO_DROP  # 10227
wxEVT_STC_DRAG_OVER = stc.wxEVT_STC_DRAG_OVER  # 10226
wxEVT_STC_DWELLEND = stc.wxEVT_STC_DWELLEND  # 10224
wxEVT_STC_DWELLSTART = stc.wxEVT_STC_DWELLSTART  # 10223
wxEVT_STC_KEY = stc.wxEVT_STC_KEY  # 10212
wxEVT_STC_MACRORECORD = stc.wxEVT_STC_MACRORECORD  # 10216
wxEVT_STC_MARGINCLICK = stc.wxEVT_STC_MARGINCLICK  # 10217
wxEVT_STC_MODIFIED = stc.wxEVT_STC_MODIFIED  # 10215
wxEVT_STC_NEEDSHOWN = stc.wxEVT_STC_NEEDSHOWN  # 10218
wxEVT_STC_PAINTED = stc.wxEVT_STC_PAINTED  # 10220
wxEVT_STC_POSCHANGED = stc.wxEVT_STC_POSCHANGED  # 10219
wxEVT_STC_ROMODIFYATTEMPT = stc.wxEVT_STC_ROMODIFYATTEMPT  # 10211
wxEVT_STC_SAVEPOINTLEFT = stc.wxEVT_STC_SAVEPOINTLEFT  # 10210
wxEVT_STC_SAVEPOINTREACHED = stc.wxEVT_STC_SAVEPOINTREACHED  # 10209
wxEVT_STC_START_DRAG = stc.wxEVT_STC_START_DRAG  # 10225
wxEVT_STC_STYLENEEDED = stc.wxEVT_STC_STYLENEEDED  # 10207
wxEVT_STC_UPDATEUI = stc.wxEVT_STC_UPDATEUI  # 10214
wxEVT_STC_URIDROPPED = stc.wxEVT_STC_URIDROPPED  # 10222
wxEVT_STC_USERLISTSELECTION = stc.wxEVT_STC_USERLISTSELECTION  # 10221
wxEVT_STC_ZOOM = stc.wxEVT_STC_ZOOM  # 10228
wxSTC_ADA_CHARACTER = stc.wxSTC_ADA_CHARACTER  # 5
wxSTC_ADA_COMMENT = stc.wxSTC_ADA_COMMENT  # 1
wxSTC_ADA_DEFAULT = stc.wxSTC_ADA_DEFAULT  # 0
wxSTC_ADA_IDENTIFIER = stc.wxSTC_ADA_IDENTIFIER  # 7
wxSTC_ADA_NUMBER = stc.wxSTC_ADA_NUMBER  # 2
wxSTC_ADA_OPERATOR = stc.wxSTC_ADA_OPERATOR  # 6
wxSTC_ADA_STRING = stc.wxSTC_ADA_STRING  # 4
wxSTC_ADA_STRINGEOL = stc.wxSTC_ADA_STRINGEOL  # 8
wxSTC_ADA_WORD = stc.wxSTC_ADA_WORD  # 3
wxSTC_AVE_COMMENT = stc.wxSTC_AVE_COMMENT  # 1
wxSTC_AVE_DEFAULT = stc.wxSTC_AVE_DEFAULT  # 0
wxSTC_AVE_ENUM = stc.wxSTC_AVE_ENUM  # 7
wxSTC_AVE_IDENTIFIER = stc.wxSTC_AVE_IDENTIFIER  # 9
wxSTC_AVE_KEYWORD = stc.wxSTC_AVE_KEYWORD  # 4
wxSTC_AVE_NUMBER = stc.wxSTC_AVE_NUMBER  # 2
wxSTC_AVE_OPERATOR = stc.wxSTC_AVE_OPERATOR  # 10
wxSTC_AVE_STATEMENT = stc.wxSTC_AVE_STATEMENT  # 5
wxSTC_AVE_STRING = stc.wxSTC_AVE_STRING  # 6
wxSTC_AVE_STRINGEOL = stc.wxSTC_AVE_STRINGEOL  # 8
wxSTC_AVE_WORD = stc.wxSTC_AVE_WORD  # 3
wxSTC_BAAN_COMMENT = stc.wxSTC_BAAN_COMMENT  # 1
wxSTC_BAAN_COMMENTDOC = stc.wxSTC_BAAN_COMMENTDOC  # 2
wxSTC_BAAN_DEFAULT = stc.wxSTC_BAAN_DEFAULT  # 0
wxSTC_BAAN_IDENTIFIER = stc.wxSTC_BAAN_IDENTIFIER  # 8
wxSTC_BAAN_NUMBER = stc.wxSTC_BAAN_NUMBER  # 3
wxSTC_BAAN_OPERATOR = stc.wxSTC_BAAN_OPERATOR  # 7
wxSTC_BAAN_PREPROCESSOR = stc.wxSTC_BAAN_PREPROCESSOR  # 6
wxSTC_BAAN_STRING = stc.wxSTC_BAAN_STRING  # 5
wxSTC_BAAN_STRINGEOL = stc.wxSTC_BAAN_STRINGEOL  # 9
wxSTC_BAAN_WORD = stc.wxSTC_BAAN_WORD  # 4
wxSTC_BAAN_WORD2 = stc.wxSTC_BAAN_WORD2  # 10
wxSTC_BAT_COMMAND = stc.wxSTC_BAT_COMMAND  # 5
wxSTC_BAT_COMMENT = stc.wxSTC_BAT_COMMENT  # 1
wxSTC_BAT_DEFAULT = stc.wxSTC_BAT_DEFAULT  # 0
wxSTC_BAT_HIDE = stc.wxSTC_BAT_HIDE  # 4
wxSTC_BAT_IDENTIFIER = stc.wxSTC_BAT_IDENTIFIER  # 6
wxSTC_BAT_LABEL = stc.wxSTC_BAT_LABEL  # 3
wxSTC_BAT_OPERATOR = stc.wxSTC_BAT_OPERATOR  # 7
wxSTC_BAT_WORD = stc.wxSTC_BAT_WORD  # 2
wxSTC_B_COMMENT = stc.wxSTC_B_COMMENT  # 1
wxSTC_B_DATE = stc.wxSTC_B_DATE  # 8
wxSTC_B_DEFAULT = stc.wxSTC_B_DEFAULT  # 0
wxSTC_B_IDENTIFIER = stc.wxSTC_B_IDENTIFIER  # 7
wxSTC_B_KEYWORD = stc.wxSTC_B_KEYWORD  # 3
wxSTC_B_NUMBER = stc.wxSTC_B_NUMBER  # 2
wxSTC_B_OPERATOR = stc.wxSTC_B_OPERATOR  # 6
wxSTC_B_PREPROCESSOR = stc.wxSTC_B_PREPROCESSOR  # 5
wxSTC_B_STRING = stc.wxSTC_B_STRING  # 4
wxSTC_CACHE_CARET = stc.wxSTC_CACHE_CARET  # 1
wxSTC_CACHE_DOCUMENT = stc.wxSTC_CACHE_DOCUMENT  # 3
wxSTC_CACHE_NONE = stc.wxSTC_CACHE_NONE  # 0
wxSTC_CACHE_PAGE = stc.wxSTC_CACHE_PAGE  # 2
wxSTC_CARET_EVEN = stc.wxSTC_CARET_EVEN  # 8
wxSTC_CARET_JUMPS = stc.wxSTC_CARET_JUMPS  # 16
wxSTC_CARET_SLOP = stc.wxSTC_CARET_SLOP  # 1
wxSTC_CARET_STRICT = stc.wxSTC_CARET_STRICT  # 4
wxSTC_CASE_LOWER = stc.wxSTC_CASE_LOWER  # 2
wxSTC_CASE_MIXED = stc.wxSTC_CASE_MIXED  # 0
wxSTC_CASE_UPPER = stc.wxSTC_CASE_UPPER  # 1
wxSTC_CHARSET_ANSI = stc.wxSTC_CHARSET_ANSI  # 0
wxSTC_CHARSET_ARABIC = stc.wxSTC_CHARSET_ARABIC  # 178
wxSTC_CHARSET_BALTIC = stc.wxSTC_CHARSET_BALTIC  # 186
wxSTC_CHARSET_CHINESEBIG5 = stc.wxSTC_CHARSET_CHINESEBIG5  # 136
wxSTC_CHARSET_DEFAULT = stc.wxSTC_CHARSET_DEFAULT  # 1
wxSTC_CHARSET_EASTEUROPE = stc.wxSTC_CHARSET_EASTEUROPE  # 238
wxSTC_CHARSET_GB2312 = stc.wxSTC_CHARSET_GB2312  # 134
wxSTC_CHARSET_GREEK = stc.wxSTC_CHARSET_GREEK  # 161
wxSTC_CHARSET_HANGUL = stc.wxSTC_CHARSET_HANGUL  # 129
wxSTC_CHARSET_HEBREW = stc.wxSTC_CHARSET_HEBREW  # 177
wxSTC_CHARSET_JOHAB = stc.wxSTC_CHARSET_JOHAB  # 130
wxSTC_CHARSET_MAC = stc.wxSTC_CHARSET_MAC  # 77
wxSTC_CHARSET_OEM = stc.wxSTC_CHARSET_OEM  # 255
wxSTC_CHARSET_RUSSIAN = stc.wxSTC_CHARSET_RUSSIAN  # 204
wxSTC_CHARSET_SHIFTJIS = stc.wxSTC_CHARSET_SHIFTJIS  # 128
wxSTC_CHARSET_SYMBOL = stc.wxSTC_CHARSET_SYMBOL  # 2
wxSTC_CHARSET_THAI = stc.wxSTC_CHARSET_THAI  # 222
wxSTC_CHARSET_TURKISH = stc.wxSTC_CHARSET_TURKISH  # 162
wxSTC_CHARSET_VIETNAMESE = stc.wxSTC_CHARSET_VIETNAMESE  # 163
wxSTC_CMD_BACKTAB = stc.wxSTC_CMD_BACKTAB  # 2328
wxSTC_CMD_CANCEL = stc.wxSTC_CMD_CANCEL  # 2325
wxSTC_CMD_CHARLEFT = stc.wxSTC_CMD_CHARLEFT  # 2304
wxSTC_CMD_CHARLEFTEXTEND = stc.wxSTC_CMD_CHARLEFTEXTEND  # 2305
wxSTC_CMD_CHARRIGHT = stc.wxSTC_CMD_CHARRIGHT  # 2306
wxSTC_CMD_CHARRIGHTEXTEND = stc.wxSTC_CMD_CHARRIGHTEXTEND  # 2307
wxSTC_CMD_CLEAR = stc.wxSTC_CMD_CLEAR  # 2180
wxSTC_CMD_COPY = stc.wxSTC_CMD_COPY  # 2178
wxSTC_CMD_CUT = stc.wxSTC_CMD_CUT  # 2177
wxSTC_CMD_DELETEBACK = stc.wxSTC_CMD_DELETEBACK  # 2326
wxSTC_CMD_DELETEBACKNOTLINE = stc.wxSTC_CMD_DELETEBACKNOTLINE  # 2344
wxSTC_CMD_DELLINELEFT = stc.wxSTC_CMD_DELLINELEFT  # 2395
wxSTC_CMD_DELLINERIGHT = stc.wxSTC_CMD_DELLINERIGHT  # 2396
wxSTC_CMD_DELWORDLEFT = stc.wxSTC_CMD_DELWORDLEFT  # 2335
wxSTC_CMD_DELWORDRIGHT = stc.wxSTC_CMD_DELWORDRIGHT  # 2336
wxSTC_CMD_DOCUMENTEND = stc.wxSTC_CMD_DOCUMENTEND  # 2318
wxSTC_CMD_DOCUMENTENDEXTEND = stc.wxSTC_CMD_DOCUMENTENDEXTEND  # 2319
wxSTC_CMD_DOCUMENTSTART = stc.wxSTC_CMD_DOCUMENTSTART  # 2316
wxSTC_CMD_DOCUMENTSTARTEXTEND = stc.wxSTC_CMD_DOCUMENTSTARTEXTEND  # 2317
wxSTC_CMD_EDITTOGGLEOVERTYPE = stc.wxSTC_CMD_EDITTOGGLEOVERTYPE  # 2324
wxSTC_CMD_FORMFEED = stc.wxSTC_CMD_FORMFEED  # 2330
wxSTC_CMD_HOME = stc.wxSTC_CMD_HOME  # 2312
wxSTC_CMD_HOMEDISPLAY = stc.wxSTC_CMD_HOMEDISPLAY  # 2345
wxSTC_CMD_HOMEDISPLAYEXTEND = stc.wxSTC_CMD_HOMEDISPLAYEXTEND  # 2346
wxSTC_CMD_HOMEEXTEND = stc.wxSTC_CMD_HOMEEXTEND  # 2313
wxSTC_CMD_LINECUT = stc.wxSTC_CMD_LINECUT  # 2337
wxSTC_CMD_LINEDELETE = stc.wxSTC_CMD_LINEDELETE  # 2338
wxSTC_CMD_LINEDOWN = stc.wxSTC_CMD_LINEDOWN  # 2300
wxSTC_CMD_LINEDOWNEXTEND = stc.wxSTC_CMD_LINEDOWNEXTEND  # 2301
wxSTC_CMD_LINEEND = stc.wxSTC_CMD_LINEEND  # 2314
wxSTC_CMD_LINEENDDISPLAY = stc.wxSTC_CMD_LINEENDDISPLAY  # 2347
wxSTC_CMD_LINEENDDISPLAYEXTEND = stc.wxSTC_CMD_LINEENDDISPLAYEXTEND  # 2348
wxSTC_CMD_LINEENDEXTEND = stc.wxSTC_CMD_LINEENDEXTEND  # 2315
wxSTC_CMD_LINESCROLLDOWN = stc.wxSTC_CMD_LINESCROLLDOWN  # 2342
wxSTC_CMD_LINESCROLLUP = stc.wxSTC_CMD_LINESCROLLUP  # 2343
wxSTC_CMD_LINETRANSPOSE = stc.wxSTC_CMD_LINETRANSPOSE  # 2339
wxSTC_CMD_LINEUP = stc.wxSTC_CMD_LINEUP  # 2302
wxSTC_CMD_LINEUPEXTEND = stc.wxSTC_CMD_LINEUPEXTEND  # 2303
wxSTC_CMD_LOWERCASE = stc.wxSTC_CMD_LOWERCASE  # 2340
wxSTC_CMD_NEWLINE = stc.wxSTC_CMD_NEWLINE  # 2329
wxSTC_CMD_PAGEDOWN = stc.wxSTC_CMD_PAGEDOWN  # 2322
wxSTC_CMD_PAGEDOWNEXTEND = stc.wxSTC_CMD_PAGEDOWNEXTEND  # 2323
wxSTC_CMD_PAGEUP = stc.wxSTC_CMD_PAGEUP  # 2320
wxSTC_CMD_PAGEUPEXTEND = stc.wxSTC_CMD_PAGEUPEXTEND  # 2321
wxSTC_CMD_PASTE = stc.wxSTC_CMD_PASTE  # 2179
wxSTC_CMD_REDO = stc.wxSTC_CMD_REDO  # 2011
wxSTC_CMD_SELECTALL = stc.wxSTC_CMD_SELECTALL  # 2013
wxSTC_CMD_TAB = stc.wxSTC_CMD_TAB  # 2327
wxSTC_CMD_UNDO = stc.wxSTC_CMD_UNDO  # 2176
wxSTC_CMD_UPPERCASE = stc.wxSTC_CMD_UPPERCASE  # 2341
wxSTC_CMD_VCHOME = stc.wxSTC_CMD_VCHOME  # 2331
wxSTC_CMD_VCHOMEEXTEND = stc.wxSTC_CMD_VCHOMEEXTEND  # 2332
wxSTC_CMD_WORDLEFT = stc.wxSTC_CMD_WORDLEFT  # 2308
wxSTC_CMD_WORDLEFTEXTEND = stc.wxSTC_CMD_WORDLEFTEXTEND  # 2309
wxSTC_CMD_WORDPARTLEFT = stc.wxSTC_CMD_WORDPARTLEFT  # 2390
wxSTC_CMD_WORDPARTLEFTEXTEND = stc.wxSTC_CMD_WORDPARTLEFTEXTEND  # 2391
wxSTC_CMD_WORDPARTRIGHT = stc.wxSTC_CMD_WORDPARTRIGHT  # 2392
wxSTC_CMD_WORDPARTRIGHTEXTEND = stc.wxSTC_CMD_WORDPARTRIGHTEXTEND  # 2393
wxSTC_CMD_WORDRIGHT = stc.wxSTC_CMD_WORDRIGHT  # 2310
wxSTC_CMD_WORDRIGHTEXTEND = stc.wxSTC_CMD_WORDRIGHTEXTEND  # 2311
wxSTC_CMD_ZOOMIN = stc.wxSTC_CMD_ZOOMIN  # 2333
wxSTC_CMD_ZOOMOUT = stc.wxSTC_CMD_ZOOMOUT  # 2334
wxSTC_CONF_COMMENT = stc.wxSTC_CONF_COMMENT  # 1
wxSTC_CONF_DEFAULT = stc.wxSTC_CONF_DEFAULT  # 0
wxSTC_CONF_DIRECTIVE = stc.wxSTC_CONF_DIRECTIVE  # 9
wxSTC_CONF_EXTENSION = stc.wxSTC_CONF_EXTENSION  # 4
wxSTC_CONF_IDENTIFIER = stc.wxSTC_CONF_IDENTIFIER  # 3
wxSTC_CONF_IP = stc.wxSTC_CONF_IP  # 8
wxSTC_CONF_NUMBER = stc.wxSTC_CONF_NUMBER  # 2
wxSTC_CONF_OPERATOR = stc.wxSTC_CONF_OPERATOR  # 7
wxSTC_CONF_PARAMETER = stc.wxSTC_CONF_PARAMETER  # 5
wxSTC_CONF_STRING = stc.wxSTC_CONF_STRING  # 6
wxSTC_CP_UTF8 = stc.wxSTC_CP_UTF8  # 65001
wxSTC_CURSORNORMAL = stc.wxSTC_CURSORNORMAL  # -1
wxSTC_CURSORWAIT = stc.wxSTC_CURSORWAIT  # 3
wxSTC_C_CHARACTER = stc.wxSTC_C_CHARACTER  # 7
wxSTC_C_COMMENT = stc.wxSTC_C_COMMENT  # 1
wxSTC_C_COMMENTDOC = stc.wxSTC_C_COMMENTDOC  # 3
wxSTC_C_COMMENTDOCKEYWORD = stc.wxSTC_C_COMMENTDOCKEYWORD  # 17
wxSTC_C_COMMENTDOCKEYWORDERROR = stc.wxSTC_C_COMMENTDOCKEYWORDERROR  # 18
wxSTC_C_COMMENTLINE = stc.wxSTC_C_COMMENTLINE  # 2
wxSTC_C_COMMENTLINEDOC = stc.wxSTC_C_COMMENTLINEDOC  # 15
wxSTC_C_DEFAULT = stc.wxSTC_C_DEFAULT  # 0
wxSTC_C_IDENTIFIER = stc.wxSTC_C_IDENTIFIER  # 11
wxSTC_C_NUMBER = stc.wxSTC_C_NUMBER  # 4
wxSTC_C_OPERATOR = stc.wxSTC_C_OPERATOR  # 10
wxSTC_C_PREPROCESSOR = stc.wxSTC_C_PREPROCESSOR  # 9
wxSTC_C_REGEX = stc.wxSTC_C_REGEX  # 14
wxSTC_C_STRING = stc.wxSTC_C_STRING  # 6
wxSTC_C_STRINGEOL = stc.wxSTC_C_STRINGEOL  # 12
wxSTC_C_UUID = stc.wxSTC_C_UUID  # 8
wxSTC_C_VERBATIM = stc.wxSTC_C_VERBATIM  # 13
wxSTC_C_WORD = stc.wxSTC_C_WORD  # 5
wxSTC_C_WORD2 = stc.wxSTC_C_WORD2  # 16
wxSTC_DIFF_ADDED = stc.wxSTC_DIFF_ADDED  # 6
wxSTC_DIFF_COMMAND = stc.wxSTC_DIFF_COMMAND  # 2
wxSTC_DIFF_COMMENT = stc.wxSTC_DIFF_COMMENT  # 1
wxSTC_DIFF_DEFAULT = stc.wxSTC_DIFF_DEFAULT  # 0
wxSTC_DIFF_DELETED = stc.wxSTC_DIFF_DELETED  # 5
wxSTC_DIFF_HEADER = stc.wxSTC_DIFF_HEADER  # 3
wxSTC_DIFF_POSITION = stc.wxSTC_DIFF_POSITION  # 4
wxSTC_EDGE_BACKGROUND = stc.wxSTC_EDGE_BACKGROUND  # 2
wxSTC_EDGE_LINE = stc.wxSTC_EDGE_LINE  # 1
wxSTC_EDGE_NONE = stc.wxSTC_EDGE_NONE  # 0
wxSTC_EIFFEL_CHARACTER = stc.wxSTC_EIFFEL_CHARACTER  # 5
wxSTC_EIFFEL_COMMENTLINE = stc.wxSTC_EIFFEL_COMMENTLINE  # 1
wxSTC_EIFFEL_DEFAULT = stc.wxSTC_EIFFEL_DEFAULT  # 0
wxSTC_EIFFEL_IDENTIFIER = stc.wxSTC_EIFFEL_IDENTIFIER  # 7
wxSTC_EIFFEL_NUMBER = stc.wxSTC_EIFFEL_NUMBER  # 2
wxSTC_EIFFEL_OPERATOR = stc.wxSTC_EIFFEL_OPERATOR  # 6
wxSTC_EIFFEL_STRING = stc.wxSTC_EIFFEL_STRING  # 4
wxSTC_EIFFEL_STRINGEOL = stc.wxSTC_EIFFEL_STRINGEOL  # 8
wxSTC_EIFFEL_WORD = stc.wxSTC_EIFFEL_WORD  # 3
wxSTC_EOL_CR = stc.wxSTC_EOL_CR  # 1
wxSTC_EOL_CRLF = stc.wxSTC_EOL_CRLF  # 0
wxSTC_EOL_LF = stc.wxSTC_EOL_LF  # 2
wxSTC_ERR_BORLAND = stc.wxSTC_ERR_BORLAND  # 5
wxSTC_ERR_CMD = stc.wxSTC_ERR_CMD  # 4
wxSTC_ERR_CTAG = stc.wxSTC_ERR_CTAG  # 9
wxSTC_ERR_DEFAULT = stc.wxSTC_ERR_DEFAULT  # 0
wxSTC_ERR_DIFF_ADDITION = stc.wxSTC_ERR_DIFF_ADDITION  # 11
wxSTC_ERR_DIFF_CHANGED = stc.wxSTC_ERR_DIFF_CHANGED  # 10
wxSTC_ERR_DIFF_DELETION = stc.wxSTC_ERR_DIFF_DELETION  # 12
wxSTC_ERR_DIFF_MESSAGE = stc.wxSTC_ERR_DIFF_MESSAGE  # 13
wxSTC_ERR_GCC = stc.wxSTC_ERR_GCC  # 2
wxSTC_ERR_LUA = stc.wxSTC_ERR_LUA  # 8
wxSTC_ERR_MS = stc.wxSTC_ERR_MS  # 3
wxSTC_ERR_NET = stc.wxSTC_ERR_NET  # 7
wxSTC_ERR_PERL = stc.wxSTC_ERR_PERL  # 6
wxSTC_ERR_PYTHON = stc.wxSTC_ERR_PYTHON  # 1
wxSTC_FIND_MATCHCASE = stc.wxSTC_FIND_MATCHCASE  # 4
wxSTC_FIND_REGEXP = stc.wxSTC_FIND_REGEXP  # 2097152
wxSTC_FIND_WHOLEWORD = stc.wxSTC_FIND_WHOLEWORD  # 2
wxSTC_FIND_WORDSTART = stc.wxSTC_FIND_WORDSTART  # 1048576
wxSTC_FOLDLEVELBASE = stc.wxSTC_FOLDLEVELBASE  # 1024
wxSTC_FOLDLEVELHEADERFLAG = stc.wxSTC_FOLDLEVELHEADERFLAG  # 8192
wxSTC_FOLDLEVELNUMBERMASK = stc.wxSTC_FOLDLEVELNUMBERMASK  # 4095
wxSTC_FOLDLEVELWHITEFLAG = stc.wxSTC_FOLDLEVELWHITEFLAG  # 4096
wxSTC_HBA_COMMENTLINE = stc.wxSTC_HBA_COMMENTLINE  # 82
wxSTC_HBA_DEFAULT = stc.wxSTC_HBA_DEFAULT  # 81
wxSTC_HBA_IDENTIFIER = stc.wxSTC_HBA_IDENTIFIER  # 86
wxSTC_HBA_NUMBER = stc.wxSTC_HBA_NUMBER  # 83
wxSTC_HBA_START = stc.wxSTC_HBA_START  # 80
wxSTC_HBA_STRING = stc.wxSTC_HBA_STRING  # 85
wxSTC_HBA_STRINGEOL = stc.wxSTC_HBA_STRINGEOL  # 87
wxSTC_HBA_WORD = stc.wxSTC_HBA_WORD  # 84
wxSTC_HB_COMMENTLINE = stc.wxSTC_HB_COMMENTLINE  # 72
wxSTC_HB_DEFAULT = stc.wxSTC_HB_DEFAULT  # 71
wxSTC_HB_IDENTIFIER = stc.wxSTC_HB_IDENTIFIER  # 76
wxSTC_HB_NUMBER = stc.wxSTC_HB_NUMBER  # 73
wxSTC_HB_START = stc.wxSTC_HB_START  # 70
wxSTC_HB_STRING = stc.wxSTC_HB_STRING  # 75
wxSTC_HB_STRINGEOL = stc.wxSTC_HB_STRINGEOL  # 77
wxSTC_HB_WORD = stc.wxSTC_HB_WORD  # 74
wxSTC_HJA_COMMENT = stc.wxSTC_HJA_COMMENT  # 57
wxSTC_HJA_COMMENTDOC = stc.wxSTC_HJA_COMMENTDOC  # 59
wxSTC_HJA_COMMENTLINE = stc.wxSTC_HJA_COMMENTLINE  # 58
wxSTC_HJA_DEFAULT = stc.wxSTC_HJA_DEFAULT  # 56
wxSTC_HJA_DOUBLESTRING = stc.wxSTC_HJA_DOUBLESTRING  # 63
wxSTC_HJA_KEYWORD = stc.wxSTC_HJA_KEYWORD  # 62
wxSTC_HJA_NUMBER = stc.wxSTC_HJA_NUMBER  # 60
wxSTC_HJA_REGEX = stc.wxSTC_HJA_REGEX  # 67
wxSTC_HJA_SINGLESTRING = stc.wxSTC_HJA_SINGLESTRING  # 64
wxSTC_HJA_START = stc.wxSTC_HJA_START  # 55
wxSTC_HJA_STRINGEOL = stc.wxSTC_HJA_STRINGEOL  # 66
wxSTC_HJA_SYMBOLS = stc.wxSTC_HJA_SYMBOLS  # 65
wxSTC_HJA_WORD = stc.wxSTC_HJA_WORD  # 61
wxSTC_HJ_COMMENT = stc.wxSTC_HJ_COMMENT  # 42
wxSTC_HJ_COMMENTDOC = stc.wxSTC_HJ_COMMENTDOC  # 44
wxSTC_HJ_COMMENTLINE = stc.wxSTC_HJ_COMMENTLINE  # 43
wxSTC_HJ_DEFAULT = stc.wxSTC_HJ_DEFAULT  # 41
wxSTC_HJ_DOUBLESTRING = stc.wxSTC_HJ_DOUBLESTRING  # 48
wxSTC_HJ_KEYWORD = stc.wxSTC_HJ_KEYWORD  # 47
wxSTC_HJ_NUMBER = stc.wxSTC_HJ_NUMBER  # 45
wxSTC_HJ_REGEX = stc.wxSTC_HJ_REGEX  # 52
wxSTC_HJ_SINGLESTRING = stc.wxSTC_HJ_SINGLESTRING  # 49
wxSTC_HJ_START = stc.wxSTC_HJ_START  # 40
wxSTC_HJ_STRINGEOL = stc.wxSTC_HJ_STRINGEOL  # 51
wxSTC_HJ_SYMBOLS = stc.wxSTC_HJ_SYMBOLS  # 50
wxSTC_HJ_WORD = stc.wxSTC_HJ_WORD  # 46
wxSTC_HPA_CHARACTER = stc.wxSTC_HPA_CHARACTER  # 110
wxSTC_HPA_CLASSNAME = stc.wxSTC_HPA_CLASSNAME  # 114
wxSTC_HPA_COMMENTLINE = stc.wxSTC_HPA_COMMENTLINE  # 107
wxSTC_HPA_DEFAULT = stc.wxSTC_HPA_DEFAULT  # 106
wxSTC_HPA_DEFNAME = stc.wxSTC_HPA_DEFNAME  # 115
wxSTC_HPA_IDENTIFIER = stc.wxSTC_HPA_IDENTIFIER  # 117
wxSTC_HPA_NUMBER = stc.wxSTC_HPA_NUMBER  # 108
wxSTC_HPA_OPERATOR = stc.wxSTC_HPA_OPERATOR  # 116
wxSTC_HPA_START = stc.wxSTC_HPA_START  # 105
wxSTC_HPA_STRING = stc.wxSTC_HPA_STRING  # 109
wxSTC_HPA_TRIPLE = stc.wxSTC_HPA_TRIPLE  # 112
wxSTC_HPA_TRIPLEDOUBLE = stc.wxSTC_HPA_TRIPLEDOUBLE  # 113
wxSTC_HPA_WORD = stc.wxSTC_HPA_WORD  # 111
wxSTC_HPHP_COMMENT = stc.wxSTC_HPHP_COMMENT  # 124
wxSTC_HPHP_COMMENTLINE = stc.wxSTC_HPHP_COMMENTLINE  # 125
wxSTC_HPHP_DEFAULT = stc.wxSTC_HPHP_DEFAULT  # 118
wxSTC_HPHP_HSTRING = stc.wxSTC_HPHP_HSTRING  # 119
wxSTC_HPHP_HSTRING_VARIABLE = stc.wxSTC_HPHP_HSTRING_VARIABLE  # 126
wxSTC_HPHP_NUMBER = stc.wxSTC_HPHP_NUMBER  # 122
wxSTC_HPHP_OPERATOR = stc.wxSTC_HPHP_OPERATOR  # 127
wxSTC_HPHP_SIMPLESTRING = stc.wxSTC_HPHP_SIMPLESTRING  # 120
wxSTC_HPHP_VARIABLE = stc.wxSTC_HPHP_VARIABLE  # 123
wxSTC_HPHP_WORD = stc.wxSTC_HPHP_WORD  # 121
wxSTC_HP_CHARACTER = stc.wxSTC_HP_CHARACTER  # 95
wxSTC_HP_CLASSNAME = stc.wxSTC_HP_CLASSNAME  # 99
wxSTC_HP_COMMENTLINE = stc.wxSTC_HP_COMMENTLINE  # 92
wxSTC_HP_DEFAULT = stc.wxSTC_HP_DEFAULT  # 91
wxSTC_HP_DEFNAME = stc.wxSTC_HP_DEFNAME  # 100
wxSTC_HP_IDENTIFIER = stc.wxSTC_HP_IDENTIFIER  # 102
wxSTC_HP_NUMBER = stc.wxSTC_HP_NUMBER  # 93
wxSTC_HP_OPERATOR = stc.wxSTC_HP_OPERATOR  # 101
wxSTC_HP_START = stc.wxSTC_HP_START  # 90
wxSTC_HP_STRING = stc.wxSTC_HP_STRING  # 94
wxSTC_HP_TRIPLE = stc.wxSTC_HP_TRIPLE  # 97
wxSTC_HP_TRIPLEDOUBLE = stc.wxSTC_HP_TRIPLEDOUBLE  # 98
wxSTC_HP_WORD = stc.wxSTC_HP_WORD  # 96
wxSTC_H_ASP = stc.wxSTC_H_ASP  # 15
wxSTC_H_ASPAT = stc.wxSTC_H_ASPAT  # 16
wxSTC_H_ATTRIBUTE = stc.wxSTC_H_ATTRIBUTE  # 3
wxSTC_H_ATTRIBUTEUNKNOWN = stc.wxSTC_H_ATTRIBUTEUNKNOWN  # 4
wxSTC_H_CDATA = stc.wxSTC_H_CDATA  # 17
wxSTC_H_COMMENT = stc.wxSTC_H_COMMENT  # 9
wxSTC_H_DEFAULT = stc.wxSTC_H_DEFAULT  # 0
wxSTC_H_DOUBLESTRING = stc.wxSTC_H_DOUBLESTRING  # 6
wxSTC_H_ENTITY = stc.wxSTC_H_ENTITY  # 10
wxSTC_H_NUMBER = stc.wxSTC_H_NUMBER  # 5
wxSTC_H_OTHER = stc.wxSTC_H_OTHER  # 8
wxSTC_H_QUESTION = stc.wxSTC_H_QUESTION  # 18
wxSTC_H_SCRIPT = stc.wxSTC_H_SCRIPT  # 14
wxSTC_H_SGML_1ST_PARAM = stc.wxSTC_H_SGML_1ST_PARAM  # 23
wxSTC_H_SGML_1ST_PARAM_COMMENT = stc.wxSTC_H_SGML_1ST_PARAM_COMMENT  # 30
wxSTC_H_SGML_BLOCK_DEFAULT = stc.wxSTC_H_SGML_BLOCK_DEFAULT  # 31
wxSTC_H_SGML_COMMAND = stc.wxSTC_H_SGML_COMMAND  # 22
wxSTC_H_SGML_COMMENT = stc.wxSTC_H_SGML_COMMENT  # 29
wxSTC_H_SGML_DEFAULT = stc.wxSTC_H_SGML_DEFAULT  # 21
wxSTC_H_SGML_DOUBLESTRING = stc.wxSTC_H_SGML_DOUBLESTRING  # 24
wxSTC_H_SGML_ENTITY = stc.wxSTC_H_SGML_ENTITY  # 28
wxSTC_H_SGML_ERROR = stc.wxSTC_H_SGML_ERROR  # 26
wxSTC_H_SGML_SIMPLESTRING = stc.wxSTC_H_SGML_SIMPLESTRING  # 25
wxSTC_H_SGML_SPECIAL = stc.wxSTC_H_SGML_SPECIAL  # 27
wxSTC_H_SINGLESTRING = stc.wxSTC_H_SINGLESTRING  # 7
wxSTC_H_TAG = stc.wxSTC_H_TAG  # 1
wxSTC_H_TAGEND = stc.wxSTC_H_TAGEND  # 11
wxSTC_H_TAGUNKNOWN = stc.wxSTC_H_TAGUNKNOWN  # 2
wxSTC_H_VALUE = stc.wxSTC_H_VALUE  # 19
wxSTC_H_XCCOMMENT = stc.wxSTC_H_XCCOMMENT  # 20
wxSTC_H_XMLEND = stc.wxSTC_H_XMLEND  # 13
wxSTC_H_XMLSTART = stc.wxSTC_H_XMLSTART  # 12
wxSTC_INDIC0_MASK = stc.wxSTC_INDIC0_MASK  # 32
wxSTC_INDIC1_MASK = stc.wxSTC_INDIC1_MASK  # 64
wxSTC_INDIC2_MASK = stc.wxSTC_INDIC2_MASK  # 128
wxSTC_INDICS_MASK = stc.wxSTC_INDICS_MASK  # 224
wxSTC_INDIC_DIAGONAL = stc.wxSTC_INDIC_DIAGONAL  # 3
wxSTC_INDIC_MAX = stc.wxSTC_INDIC_MAX  # 7
wxSTC_INDIC_PLAIN = stc.wxSTC_INDIC_PLAIN  # 0
wxSTC_INDIC_SQUIGGLE = stc.wxSTC_INDIC_SQUIGGLE  # 1
wxSTC_INDIC_STRIKE = stc.wxSTC_INDIC_STRIKE  # 4
wxSTC_INDIC_TT = stc.wxSTC_INDIC_TT  # 2
wxSTC_INVALID_POSITION = stc.wxSTC_INVALID_POSITION  # -1
wxSTC_KEY_ADD = stc.wxSTC_KEY_ADD  # 310
wxSTC_KEY_BACK = stc.wxSTC_KEY_BACK  # 8
wxSTC_KEY_DELETE = stc.wxSTC_KEY_DELETE  # 308
wxSTC_KEY_DIVIDE = stc.wxSTC_KEY_DIVIDE  # 312
wxSTC_KEY_DOWN = stc.wxSTC_KEY_DOWN  # 300
wxSTC_KEY_END = stc.wxSTC_KEY_END  # 305
wxSTC_KEY_ESCAPE = stc.wxSTC_KEY_ESCAPE  # 7
wxSTC_KEY_HOME = stc.wxSTC_KEY_HOME  # 304
wxSTC_KEY_INSERT = stc.wxSTC_KEY_INSERT  # 309
wxSTC_KEY_LEFT = stc.wxSTC_KEY_LEFT  # 302
wxSTC_KEY_NEXT = stc.wxSTC_KEY_NEXT  # 307
wxSTC_KEY_PRIOR = stc.wxSTC_KEY_PRIOR  # 306
wxSTC_KEY_RETURN = stc.wxSTC_KEY_RETURN  # 13
wxSTC_KEY_RIGHT = stc.wxSTC_KEY_RIGHT  # 303
wxSTC_KEY_SUBTRACT = stc.wxSTC_KEY_SUBTRACT  # 311
wxSTC_KEY_TAB = stc.wxSTC_KEY_TAB  # 9
wxSTC_KEY_UP = stc.wxSTC_KEY_UP  # 301
wxSTC_LASTSTEPINUNDOREDO = stc.wxSTC_LASTSTEPINUNDOREDO  # 256
wxSTC_LEXER_START = stc.wxSTC_LEXER_START  # 4000
wxSTC_LEX_ADA = stc.wxSTC_LEX_ADA  # 20
wxSTC_LEX_ASP = stc.wxSTC_LEX_ASP  # 29
wxSTC_LEX_AUTOMATIC = stc.wxSTC_LEX_AUTOMATIC  # 1000
wxSTC_LEX_AVE = stc.wxSTC_LEX_AVE  # 19
wxSTC_LEX_BAAN = stc.wxSTC_LEX_BAAN  # 31
wxSTC_LEX_BATCH = stc.wxSTC_LEX_BATCH  # 12
wxSTC_LEX_BULLANT = stc.wxSTC_LEX_BULLANT  # 27
wxSTC_LEX_CONF = stc.wxSTC_LEX_CONF  # 17
wxSTC_LEX_CONTAINER = stc.wxSTC_LEX_CONTAINER  # 0
wxSTC_LEX_CPP = stc.wxSTC_LEX_CPP  # 3
wxSTC_LEX_DIFF = stc.wxSTC_LEX_DIFF  # 16
wxSTC_LEX_EIFFEL = stc.wxSTC_LEX_EIFFEL  # 23
wxSTC_LEX_EIFFELKW = stc.wxSTC_LEX_EIFFELKW  # 24
wxSTC_LEX_ERRORLIST = stc.wxSTC_LEX_ERRORLIST  # 10
wxSTC_LEX_HTML = stc.wxSTC_LEX_HTML  # 4
wxSTC_LEX_LATEX = stc.wxSTC_LEX_LATEX  # 14
wxSTC_LEX_LISP = stc.wxSTC_LEX_LISP  # 21
wxSTC_LEX_LUA = stc.wxSTC_LEX_LUA  # 15
wxSTC_LEX_MAKEFILE = stc.wxSTC_LEX_MAKEFILE  # 11
wxSTC_LEX_MATLAB = stc.wxSTC_LEX_MATLAB  # 32
wxSTC_LEX_NNCRONTAB = stc.wxSTC_LEX_NNCRONTAB  # 26
wxSTC_LEX_NULL = stc.wxSTC_LEX_NULL  # 1
wxSTC_LEX_PASCAL = stc.wxSTC_LEX_PASCAL  # 18
wxSTC_LEX_PERL = stc.wxSTC_LEX_PERL  # 6
wxSTC_LEX_PHP = stc.wxSTC_LEX_PHP  # 30
wxSTC_LEX_PROPERTIES = stc.wxSTC_LEX_PROPERTIES  # 9
wxSTC_LEX_PYTHON = stc.wxSTC_LEX_PYTHON  # 2
wxSTC_LEX_RUBY = stc.wxSTC_LEX_RUBY  # 22
wxSTC_LEX_SCRIPTOL = stc.wxSTC_LEX_SCRIPTOL  # 33
wxSTC_LEX_SQL = stc.wxSTC_LEX_SQL  # 7
wxSTC_LEX_TCL = stc.wxSTC_LEX_TCL  # 25
wxSTC_LEX_VB = stc.wxSTC_LEX_VB  # 8
wxSTC_LEX_VBSCRIPT = stc.wxSTC_LEX_VBSCRIPT  # 28
wxSTC_LEX_XCODE = stc.wxSTC_LEX_XCODE  # 13
wxSTC_LEX_XML = stc.wxSTC_LEX_XML  # 5
wxSTC_LISP_COMMENT = stc.wxSTC_LISP_COMMENT  # 1
wxSTC_LISP_DEFAULT = stc.wxSTC_LISP_DEFAULT  # 0
wxSTC_LISP_IDENTIFIER = stc.wxSTC_LISP_IDENTIFIER  # 9
wxSTC_LISP_KEYWORD = stc.wxSTC_LISP_KEYWORD  # 3
wxSTC_LISP_NUMBER = stc.wxSTC_LISP_NUMBER  # 2
wxSTC_LISP_OPERATOR = stc.wxSTC_LISP_OPERATOR  # 10
wxSTC_LISP_STRING = stc.wxSTC_LISP_STRING  # 6
wxSTC_LISP_STRINGEOL = stc.wxSTC_LISP_STRINGEOL  # 8
wxSTC_LUA_CHARACTER = stc.wxSTC_LUA_CHARACTER  # 7
wxSTC_LUA_COMMENT = stc.wxSTC_LUA_COMMENT  # 1
wxSTC_LUA_COMMENTDOC = stc.wxSTC_LUA_COMMENTDOC  # 3
wxSTC_LUA_COMMENTLINE = stc.wxSTC_LUA_COMMENTLINE  # 2
wxSTC_LUA_DEFAULT = stc.wxSTC_LUA_DEFAULT  # 0
wxSTC_LUA_IDENTIFIER = stc.wxSTC_LUA_IDENTIFIER  # 11
wxSTC_LUA_LITERALSTRING = stc.wxSTC_LUA_LITERALSTRING  # 8
wxSTC_LUA_NUMBER = stc.wxSTC_LUA_NUMBER  # 4
wxSTC_LUA_OPERATOR = stc.wxSTC_LUA_OPERATOR  # 10
wxSTC_LUA_PREPROCESSOR = stc.wxSTC_LUA_PREPROCESSOR  # 9
wxSTC_LUA_STRING = stc.wxSTC_LUA_STRING  # 6
wxSTC_LUA_STRINGEOL = stc.wxSTC_LUA_STRINGEOL  # 12
wxSTC_LUA_WORD = stc.wxSTC_LUA_WORD  # 5
wxSTC_LUA_WORD2 = stc.wxSTC_LUA_WORD2  # 13
wxSTC_LUA_WORD3 = stc.wxSTC_LUA_WORD3  # 14
wxSTC_LUA_WORD4 = stc.wxSTC_LUA_WORD4  # 15
wxSTC_LUA_WORD5 = stc.wxSTC_LUA_WORD5  # 16
wxSTC_LUA_WORD6 = stc.wxSTC_LUA_WORD6  # 17
wxSTC_L_COMMAND = stc.wxSTC_L_COMMAND  # 1
wxSTC_L_COMMENT = stc.wxSTC_L_COMMENT  # 4
wxSTC_L_DEFAULT = stc.wxSTC_L_DEFAULT  # 0
wxSTC_L_MATH = stc.wxSTC_L_MATH  # 3
wxSTC_L_TAG = stc.wxSTC_L_TAG  # 2
wxSTC_MAKE_COMMENT = stc.wxSTC_MAKE_COMMENT  # 1
wxSTC_MAKE_DEFAULT = stc.wxSTC_MAKE_DEFAULT  # 0
wxSTC_MAKE_IDENTIFIER = stc.wxSTC_MAKE_IDENTIFIER  # 3
wxSTC_MAKE_IDEOL = stc.wxSTC_MAKE_IDEOL  # 9
wxSTC_MAKE_OPERATOR = stc.wxSTC_MAKE_OPERATOR  # 4
wxSTC_MAKE_PREPROCESSOR = stc.wxSTC_MAKE_PREPROCESSOR  # 2
wxSTC_MAKE_TARGET = stc.wxSTC_MAKE_TARGET  # 5
wxSTC_MARGIN_NUMBER = stc.wxSTC_MARGIN_NUMBER  # 1
wxSTC_MARGIN_SYMBOL = stc.wxSTC_MARGIN_SYMBOL  # 0
wxSTC_MARKER_MAX = stc.wxSTC_MARKER_MAX  # 31
wxSTC_MARKNUM_FOLDER = stc.wxSTC_MARKNUM_FOLDER  # 30
wxSTC_MARKNUM_FOLDEREND = stc.wxSTC_MARKNUM_FOLDEREND  # 25
wxSTC_MARKNUM_FOLDERMIDTAIL = stc.wxSTC_MARKNUM_FOLDERMIDTAIL  # 27
wxSTC_MARKNUM_FOLDEROPEN = stc.wxSTC_MARKNUM_FOLDEROPEN  # 31
wxSTC_MARKNUM_FOLDEROPENMID = stc.wxSTC_MARKNUM_FOLDEROPENMID  # 26
wxSTC_MARKNUM_FOLDERSUB = stc.wxSTC_MARKNUM_FOLDERSUB  # 29
wxSTC_MARKNUM_FOLDERTAIL = stc.wxSTC_MARKNUM_FOLDERTAIL  # 28
wxSTC_MARK_ARROW = stc.wxSTC_MARK_ARROW  # 2
wxSTC_MARK_ARROWDOWN = stc.wxSTC_MARK_ARROWDOWN  # 6
wxSTC_MARK_ARROWS = stc.wxSTC_MARK_ARROWS  # 24
wxSTC_MARK_BACKGROUND = stc.wxSTC_MARK_BACKGROUND  # 22
wxSTC_MARK_BOXMINUS = stc.wxSTC_MARK_BOXMINUS  # 14
wxSTC_MARK_BOXMINUSCONNECTED = stc.wxSTC_MARK_BOXMINUSCONNECTED  # 15
wxSTC_MARK_BOXPLUS = stc.wxSTC_MARK_BOXPLUS  # 12
wxSTC_MARK_BOXPLUSCONNECTED = stc.wxSTC_MARK_BOXPLUSCONNECTED  # 13
wxSTC_MARK_CHARACTER = stc.wxSTC_MARK_CHARACTER  # 10000
wxSTC_MARK_CIRCLE = stc.wxSTC_MARK_CIRCLE  # 0
wxSTC_MARK_CIRCLEMINUS = stc.wxSTC_MARK_CIRCLEMINUS  # 20
wxSTC_MARK_CIRCLEMINUSCONNECTED = stc.wxSTC_MARK_CIRCLEMINUSCONNECTED  # 21
wxSTC_MARK_CIRCLEPLUS = stc.wxSTC_MARK_CIRCLEPLUS  # 18
wxSTC_MARK_CIRCLEPLUSCONNECTED = stc.wxSTC_MARK_CIRCLEPLUSCONNECTED  # 19
wxSTC_MARK_DOTDOTDOT = stc.wxSTC_MARK_DOTDOTDOT  # 23
wxSTC_MARK_EMPTY = stc.wxSTC_MARK_EMPTY  # 5
wxSTC_MARK_LCORNER = stc.wxSTC_MARK_LCORNER  # 10
wxSTC_MARK_LCORNERCURVE = stc.wxSTC_MARK_LCORNERCURVE  # 16
wxSTC_MARK_MINUS = stc.wxSTC_MARK_MINUS  # 7
wxSTC_MARK_PLUS = stc.wxSTC_MARK_PLUS  # 8
wxSTC_MARK_ROUNDRECT = stc.wxSTC_MARK_ROUNDRECT  # 1
wxSTC_MARK_SHORTARROW = stc.wxSTC_MARK_SHORTARROW  # 4
wxSTC_MARK_SMALLRECT = stc.wxSTC_MARK_SMALLRECT  # 3
wxSTC_MARK_TCORNER = stc.wxSTC_MARK_TCORNER  # 11
wxSTC_MARK_TCORNERCURVE = stc.wxSTC_MARK_TCORNERCURVE  # 17
wxSTC_MARK_VLINE = stc.wxSTC_MARK_VLINE  # 9
wxSTC_MASK_FOLDERS = stc.wxSTC_MASK_FOLDERS  # -33554432
wxSTC_MATLAB_COMMAND = stc.wxSTC_MATLAB_COMMAND  # 2
wxSTC_MATLAB_COMMENT = stc.wxSTC_MATLAB_COMMENT  # 1
wxSTC_MATLAB_DEFAULT = stc.wxSTC_MATLAB_DEFAULT  # 0
wxSTC_MATLAB_IDENTIFIER = stc.wxSTC_MATLAB_IDENTIFIER  # 7
wxSTC_MATLAB_KEYWORD = stc.wxSTC_MATLAB_KEYWORD  # 4
wxSTC_MATLAB_NUMBER = stc.wxSTC_MATLAB_NUMBER  # 3
wxSTC_MATLAB_OPERATOR = stc.wxSTC_MATLAB_OPERATOR  # 6
wxSTC_MATLAB_STRING = stc.wxSTC_MATLAB_STRING  # 5
wxSTC_MODEVENTMASKALL = stc.wxSTC_MODEVENTMASKALL  # 3959
wxSTC_MOD_BEFOREDELETE = stc.wxSTC_MOD_BEFOREDELETE  # 2048
wxSTC_MOD_BEFOREINSERT = stc.wxSTC_MOD_BEFOREINSERT  # 1024
wxSTC_MOD_CHANGEFOLD = stc.wxSTC_MOD_CHANGEFOLD  # 8
wxSTC_MOD_CHANGEMARKER = stc.wxSTC_MOD_CHANGEMARKER  # 512
wxSTC_MOD_CHANGESTYLE = stc.wxSTC_MOD_CHANGESTYLE  # 4
wxSTC_MOD_DELETETEXT = stc.wxSTC_MOD_DELETETEXT  # 2
wxSTC_MOD_INSERTTEXT = stc.wxSTC_MOD_INSERTTEXT  # 1
wxSTC_NNCRONTAB_ASTERISK = stc.wxSTC_NNCRONTAB_ASTERISK  # 6
wxSTC_NNCRONTAB_COMMENT = stc.wxSTC_NNCRONTAB_COMMENT  # 1
wxSTC_NNCRONTAB_DEFAULT = stc.wxSTC_NNCRONTAB_DEFAULT  # 0
wxSTC_NNCRONTAB_ENVIRONMENT = stc.wxSTC_NNCRONTAB_ENVIRONMENT  # 9
wxSTC_NNCRONTAB_IDENTIFIER = stc.wxSTC_NNCRONTAB_IDENTIFIER  # 10
wxSTC_NNCRONTAB_KEYWORD = stc.wxSTC_NNCRONTAB_KEYWORD  # 4
wxSTC_NNCRONTAB_MODIFIER = stc.wxSTC_NNCRONTAB_MODIFIER  # 5
wxSTC_NNCRONTAB_NUMBER = stc.wxSTC_NNCRONTAB_NUMBER  # 7
wxSTC_NNCRONTAB_SECTION = stc.wxSTC_NNCRONTAB_SECTION  # 3
wxSTC_NNCRONTAB_STRING = stc.wxSTC_NNCRONTAB_STRING  # 8
wxSTC_NNCRONTAB_TASK = stc.wxSTC_NNCRONTAB_TASK  # 2
wxSTC_OPTIONAL_START = stc.wxSTC_OPTIONAL_START  # 3000
wxSTC_PERFORMED_REDO = stc.wxSTC_PERFORMED_REDO  # 64
wxSTC_PERFORMED_UNDO = stc.wxSTC_PERFORMED_UNDO  # 32
wxSTC_PERFORMED_USER = stc.wxSTC_PERFORMED_USER  # 16
wxSTC_PL_ARRAY = stc.wxSTC_PL_ARRAY  # 13
wxSTC_PL_BACKTICKS = stc.wxSTC_PL_BACKTICKS  # 20
wxSTC_PL_CHARACTER = stc.wxSTC_PL_CHARACTER  # 7
wxSTC_PL_COMMENTLINE = stc.wxSTC_PL_COMMENTLINE  # 2
wxSTC_PL_DATASECTION = stc.wxSTC_PL_DATASECTION  # 21
wxSTC_PL_DEFAULT = stc.wxSTC_PL_DEFAULT  # 0
wxSTC_PL_ERROR = stc.wxSTC_PL_ERROR  # 1
wxSTC_PL_HASH = stc.wxSTC_PL_HASH  # 14
wxSTC_PL_HERE_DELIM = stc.wxSTC_PL_HERE_DELIM  # 22
wxSTC_PL_HERE_Q = stc.wxSTC_PL_HERE_Q  # 23
wxSTC_PL_HERE_QQ = stc.wxSTC_PL_HERE_QQ  # 24
wxSTC_PL_HERE_QX = stc.wxSTC_PL_HERE_QX  # 25
wxSTC_PL_IDENTIFIER = stc.wxSTC_PL_IDENTIFIER  # 11
wxSTC_PL_LONGQUOTE = stc.wxSTC_PL_LONGQUOTE  # 19
wxSTC_PL_NUMBER = stc.wxSTC_PL_NUMBER  # 4
wxSTC_PL_OPERATOR = stc.wxSTC_PL_OPERATOR  # 10
wxSTC_PL_POD = stc.wxSTC_PL_POD  # 3
wxSTC_PL_PREPROCESSOR = stc.wxSTC_PL_PREPROCESSOR  # 9
wxSTC_PL_PUNCTUATION = stc.wxSTC_PL_PUNCTUATION  # 8
wxSTC_PL_REGEX = stc.wxSTC_PL_REGEX  # 17
wxSTC_PL_REGSUBST = stc.wxSTC_PL_REGSUBST  # 18
wxSTC_PL_SCALAR = stc.wxSTC_PL_SCALAR  # 12
wxSTC_PL_STRING = stc.wxSTC_PL_STRING  # 6
wxSTC_PL_STRING_Q = stc.wxSTC_PL_STRING_Q  # 26
wxSTC_PL_STRING_QQ = stc.wxSTC_PL_STRING_QQ  # 27
wxSTC_PL_STRING_QR = stc.wxSTC_PL_STRING_QR  # 29
wxSTC_PL_STRING_QW = stc.wxSTC_PL_STRING_QW  # 30
wxSTC_PL_STRING_QX = stc.wxSTC_PL_STRING_QX  # 28
wxSTC_PL_SYMBOLTABLE = stc.wxSTC_PL_SYMBOLTABLE  # 15
wxSTC_PL_WORD = stc.wxSTC_PL_WORD  # 5
wxSTC_PRINT_BLACKONWHITE = stc.wxSTC_PRINT_BLACKONWHITE  # 2
wxSTC_PRINT_COLOURONWHITE = stc.wxSTC_PRINT_COLOURONWHITE  # 3
wxSTC_PRINT_COLOURONWHITEDEFAULTBG = stc.wxSTC_PRINT_COLOURONWHITEDEFAULTBG  # 4
wxSTC_PRINT_INVERTLIGHT = stc.wxSTC_PRINT_INVERTLIGHT  # 1
wxSTC_PRINT_NORMAL = stc.wxSTC_PRINT_NORMAL  # 0
wxSTC_PROPS_ASSIGNMENT = stc.wxSTC_PROPS_ASSIGNMENT  # 3
wxSTC_PROPS_COMMENT = stc.wxSTC_PROPS_COMMENT  # 1
wxSTC_PROPS_DEFAULT = stc.wxSTC_PROPS_DEFAULT  # 0
wxSTC_PROPS_DEFVAL = stc.wxSTC_PROPS_DEFVAL  # 4
wxSTC_PROPS_SECTION = stc.wxSTC_PROPS_SECTION  # 2
wxSTC_P_CHARACTER = stc.wxSTC_P_CHARACTER  # 4
wxSTC_P_CLASSNAME = stc.wxSTC_P_CLASSNAME  # 8
wxSTC_P_COMMENTBLOCK = stc.wxSTC_P_COMMENTBLOCK  # 12
wxSTC_P_COMMENTLINE = stc.wxSTC_P_COMMENTLINE  # 1
wxSTC_P_DEFAULT = stc.wxSTC_P_DEFAULT  # 0
wxSTC_P_DEFNAME = stc.wxSTC_P_DEFNAME  # 9
wxSTC_P_IDENTIFIER = stc.wxSTC_P_IDENTIFIER  # 11
wxSTC_P_NUMBER = stc.wxSTC_P_NUMBER  # 2
wxSTC_P_OPERATOR = stc.wxSTC_P_OPERATOR  # 10
wxSTC_P_STRING = stc.wxSTC_P_STRING  # 3
wxSTC_P_STRINGEOL = stc.wxSTC_P_STRINGEOL  # 13
wxSTC_P_TRIPLE = stc.wxSTC_P_TRIPLE  # 6
wxSTC_P_TRIPLEDOUBLE = stc.wxSTC_P_TRIPLEDOUBLE  # 7
wxSTC_P_WORD = stc.wxSTC_P_WORD  # 5
wxSTC_SCMOD_ALT = stc.wxSTC_SCMOD_ALT  # 4
wxSTC_SCMOD_CTRL = stc.wxSTC_SCMOD_CTRL  # 2
wxSTC_SCMOD_SHIFT = stc.wxSTC_SCMOD_SHIFT  # 1
wxSTC_SCRIPTOL_CHARACTER = stc.wxSTC_SCRIPTOL_CHARACTER  # 7
wxSTC_SCRIPTOL_COMMENT = stc.wxSTC_SCRIPTOL_COMMENT  # 1
wxSTC_SCRIPTOL_COMMENTBASIC = stc.wxSTC_SCRIPTOL_COMMENTBASIC  # 19
wxSTC_SCRIPTOL_COMMENTDOC = stc.wxSTC_SCRIPTOL_COMMENTDOC  # 3
wxSTC_SCRIPTOL_COMMENTDOCKEYWORD = stc.wxSTC_SCRIPTOL_COMMENTDOCKEYWORD  # 17
wxSTC_SCRIPTOL_COMMENTDOCKEYWORDERROR = stc.wxSTC_SCRIPTOL_COMMENTDOCKEYWORDERROR  # 18
wxSTC_SCRIPTOL_COMMENTLINE = stc.wxSTC_SCRIPTOL_COMMENTLINE  # 2
wxSTC_SCRIPTOL_COMMENTLINEDOC = stc.wxSTC_SCRIPTOL_COMMENTLINEDOC  # 15
wxSTC_SCRIPTOL_DEFAULT = stc.wxSTC_SCRIPTOL_DEFAULT  # 0
wxSTC_SCRIPTOL_IDENTIFIER = stc.wxSTC_SCRIPTOL_IDENTIFIER  # 11
wxSTC_SCRIPTOL_NUMBER = stc.wxSTC_SCRIPTOL_NUMBER  # 4
wxSTC_SCRIPTOL_OPERATOR = stc.wxSTC_SCRIPTOL_OPERATOR  # 10
wxSTC_SCRIPTOL_PREPROCESSOR = stc.wxSTC_SCRIPTOL_PREPROCESSOR  # 9
wxSTC_SCRIPTOL_REGEX = stc.wxSTC_SCRIPTOL_REGEX  # 14
wxSTC_SCRIPTOL_STRING = stc.wxSTC_SCRIPTOL_STRING  # 6
wxSTC_SCRIPTOL_STRINGEOL = stc.wxSTC_SCRIPTOL_STRINGEOL  # 12
wxSTC_SCRIPTOL_UUID = stc.wxSTC_SCRIPTOL_UUID  # 8
wxSTC_SCRIPTOL_VERBATIM = stc.wxSTC_SCRIPTOL_VERBATIM  # 13
wxSTC_SCRIPTOL_WORD = stc.wxSTC_SCRIPTOL_WORD  # 5
wxSTC_SCRIPTOL_WORD2 = stc.wxSTC_SCRIPTOL_WORD2  # 16
wxSTC_START = stc.wxSTC_START  # 2000
wxSTC_STYLE_BRACEBAD = stc.wxSTC_STYLE_BRACEBAD  # 35
wxSTC_STYLE_BRACELIGHT = stc.wxSTC_STYLE_BRACELIGHT  # 34
wxSTC_STYLE_CONTROLCHAR = stc.wxSTC_STYLE_CONTROLCHAR  # 36
wxSTC_STYLE_DEFAULT = stc.wxSTC_STYLE_DEFAULT  # 32
wxSTC_STYLE_INDENTGUIDE = stc.wxSTC_STYLE_INDENTGUIDE  # 37
wxSTC_STYLE_LASTPREDEFINED = stc.wxSTC_STYLE_LASTPREDEFINED  # 39
wxSTC_STYLE_LINENUMBER = stc.wxSTC_STYLE_LINENUMBER  # 33
wxSTC_STYLE_MAX = stc.wxSTC_STYLE_MAX  # 127
wxSTC_TIME_FOREVER = stc.wxSTC_TIME_FOREVER  # 10000000
wxSTC_USE_POPUP = stc.wxSTC_USE_POPUP  # 1
wxSTC_VISIBLE_SLOP = stc.wxSTC_VISIBLE_SLOP  # 1
wxSTC_VISIBLE_STRICT = stc.wxSTC_VISIBLE_STRICT  # 4
wxSTC_WRAP_NONE = stc.wxSTC_WRAP_NONE  # 0
wxSTC_WRAP_WORD = stc.wxSTC_WRAP_WORD  # 1
wxSTC_WS_INVISIBLE = stc.wxSTC_WS_INVISIBLE  # 0
wxSTC_WS_VISIBLEAFTERINDENT = stc.wxSTC_WS_VISIBLEAFTERINDENT  # 2
wxSTC_WS_VISIBLEALWAYS = stc.wxSTC_WS_VISIBLEALWAYS  # 1
# End generated constants.



"""Shell menu mixin shared by shell and crust."""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: shellmenu.py,v 1.1.2.3 2003/03/11 19:37:33 RD Exp $"
__revision__ = "$Revision: 1.1.2.3 $"[11:-2]

from wxPython import wx
import sys
from version import VERSION

try:
    True
except NameError:
    True = 1==1
    False = 1==0

ID_AUTOCOMP = wx.wxNewId()
ID_AUTOCOMP_SHOW = wx.wxNewId()
ID_AUTOCOMP_INCLUDE_MAGIC = wx.wxNewId()
ID_AUTOCOMP_INCLUDE_SINGLE = wx.wxNewId()
ID_AUTOCOMP_INCLUDE_DOUBLE = wx.wxNewId()
ID_CALLTIPS = wx.wxNewId()
ID_CALLTIPS_SHOW = wx.wxNewId()
ID_COPY_PLUS = wx.wxNewId()
ID_PASTE_PLUS = wx.wxNewId()
ID_WRAP = wx.wxNewId()


class ShellMenu:
    """Mixin class to add standard menu items."""

    def createMenus(self):
        m = self.fileMenu = wx.wxMenu()
        m.AppendSeparator()
        m.Append(wx.wxID_EXIT, 'E&xit', 'Exit PyCrust')

        m = self.editMenu = wx.wxMenu()
        m.Append(wx.wxID_UNDO, '&Undo \tCtrl+Z',
                 'Undo the last action')
        m.Append(wx.wxID_REDO, '&Redo \tCtrl+Y',
                 'Redo the last undone action')
        m.AppendSeparator()
        m.Append(wx.wxID_CUT, 'Cu&t \tCtrl+X',
                 'Cut the selection')
        m.Append(wx.wxID_COPY, '&Copy \tCtrl+C',
                 'Copy the selection - removing prompts')
        m.Append(ID_COPY_PLUS, 'Cop&y Plus \tCtrl+Shift+C',
                 'Copy the selection - retaining prompts')
        m.Append(wx.wxID_PASTE, '&Paste \tCtrl+V', 'Paste')
        m.Append(ID_PASTE_PLUS, 'Past&e Plus \tCtrl+Shift+V',
                 'Paste and run commands')
        m.AppendSeparator()
        m.Append(wx.wxID_CLEAR, 'Cle&ar',
                 'Delete the selection')
        m.Append(wx.wxID_SELECTALL, 'Select A&ll \tCtrl+A',
                 'Select all text')

        m = self.autocompMenu = wx.wxMenu()
        m.Append(ID_AUTOCOMP_SHOW, 'Show Auto Completion',
                 'Show auto completion during dot syntax', 1)
        m.Append(ID_AUTOCOMP_INCLUDE_MAGIC, 'Include Magic Attributes',
                 'Include attributes visible to __getattr__ and __setattr__',
                 1)
        m.Append(ID_AUTOCOMP_INCLUDE_SINGLE, 'Include Single Underscores',
                 'Include attibutes prefixed by a single underscore', 1)
        m.Append(ID_AUTOCOMP_INCLUDE_DOUBLE, 'Include Double Underscores',
                 'Include attibutes prefixed by a double underscore', 1)

        m = self.calltipsMenu = wx.wxMenu()
        m.Append(ID_CALLTIPS_SHOW, 'Show Call Tips',
                 'Show call tips with argument specifications', 1)

        m = self.optionsMenu = wx.wxMenu()
        m.AppendMenu(ID_AUTOCOMP, '&Auto Completion', self.autocompMenu,
                     'Auto Completion Options')
        m.AppendMenu(ID_CALLTIPS, '&Call Tips', self.calltipsMenu,
                     'Call Tip Options')
        m.Append(ID_WRAP, '&Wrap Lines',
                 'Wrap lines at right edge', 1)

        m = self.helpMenu = wx.wxMenu()
        m.AppendSeparator()
        m.Append(wx.wxID_ABOUT, '&About...', 'About PyCrust')

        b = self.menuBar = wx.wxMenuBar()
        b.Append(self.fileMenu, '&File')
        b.Append(self.editMenu, '&Edit')
        b.Append(self.optionsMenu, '&Options')
        b.Append(self.helpMenu, '&Help')
        self.SetMenuBar(b)

        wx.EVT_MENU(self, wx.wxID_EXIT, self.OnExit)
        wx.EVT_MENU(self, wx.wxID_UNDO, self.OnUndo)
        wx.EVT_MENU(self, wx.wxID_REDO, self.OnRedo)
        wx.EVT_MENU(self, wx.wxID_CUT, self.OnCut)
        wx.EVT_MENU(self, wx.wxID_COPY, self.OnCopy)
        wx.EVT_MENU(self, ID_COPY_PLUS, self.OnCopyPlus)
        wx.EVT_MENU(self, wx.wxID_PASTE, self.OnPaste)
        wx.EVT_MENU(self, ID_PASTE_PLUS, self.OnPastePlus)
        wx.EVT_MENU(self, wx.wxID_CLEAR, self.OnClear)
        wx.EVT_MENU(self, wx.wxID_SELECTALL, self.OnSelectAll)
        wx.EVT_MENU(self, wx.wxID_ABOUT, self.OnAbout)
        wx.EVT_MENU(self, ID_AUTOCOMP_SHOW,
                    self.OnAutoCompleteShow)
        wx.EVT_MENU(self, ID_AUTOCOMP_INCLUDE_MAGIC,
                    self.OnAutoCompleteIncludeMagic)
        wx.EVT_MENU(self, ID_AUTOCOMP_INCLUDE_SINGLE,
                    self.OnAutoCompleteIncludeSingle)
        wx.EVT_MENU(self, ID_AUTOCOMP_INCLUDE_DOUBLE,
                    self.OnAutoCompleteIncludeDouble)
        wx.EVT_MENU(self, ID_CALLTIPS_SHOW,
                    self.OnCallTipsShow)
        wx.EVT_MENU(self, ID_WRAP, self.OnWrap)

        wx.EVT_UPDATE_UI(self, wx.wxID_UNDO, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, wx.wxID_REDO, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, wx.wxID_CUT, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, wx.wxID_COPY, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, ID_COPY_PLUS, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, wx.wxID_PASTE, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, ID_PASTE_PLUS, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, wx.wxID_CLEAR, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, ID_AUTOCOMP_SHOW, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, ID_AUTOCOMP_INCLUDE_MAGIC, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, ID_AUTOCOMP_INCLUDE_SINGLE, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, ID_AUTOCOMP_INCLUDE_DOUBLE, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, ID_CALLTIPS_SHOW, self.OnUpdateMenu)
        wx.EVT_UPDATE_UI(self, ID_WRAP, self.OnUpdateMenu)

    def OnExit(self, event):
        self.Close(True)

    def OnUndo(self, event):
        self.shell.Undo()

    def OnRedo(self, event):
        self.shell.Redo()

    def OnCut(self, event):
        self.shell.Cut()

    def OnCopy(self, event):
        self.shell.Copy()

    def OnCopyPlus(self, event):
        self.shell.CopyWithPrompts()

    def OnPaste(self, event):
        self.shell.Paste()

    def OnPastePlus(self, event):
        self.shell.PasteAndRun()

    def OnClear(self, event):
        self.shell.Clear()

    def OnSelectAll(self, event):
        self.shell.SelectAll()

    def OnAbout(self, event):
        """Display an About PyCrust window."""
        title = 'About PyCrust'
        text = 'PyCrust %s\n\n' % VERSION + \
               'Yet another Python shell, only flakier.\n\n' + \
               'Half-baked by Patrick K. O\'Brien,\n' + \
               'the other half is still in the oven.\n\n' + \
               'Shell Revision: %s\n' % self.shell.revision + \
               'Interpreter Revision: %s\n\n' % self.shell.interp.revision + \
               'Python Version: %s\n' % sys.version.split()[0] + \
               'wxPython Version: %s\n' % wx.__version__ + \
               'Platform: %s\n' % sys.platform
        dialog = wx.wxMessageDialog(self, text, title,
                                    wx.wxOK | wx.wxICON_INFORMATION)
        dialog.ShowModal()
        dialog.Destroy()

    def OnAutoCompleteShow(self, event):
        self.shell.autoComplete = event.IsChecked()

    def OnAutoCompleteIncludeMagic(self, event):
        self.shell.autoCompleteIncludeMagic = event.IsChecked()

    def OnAutoCompleteIncludeSingle(self, event):
        self.shell.autoCompleteIncludeSingle = event.IsChecked()

    def OnAutoCompleteIncludeDouble(self, event):
        self.shell.autoCompleteIncludeDouble = event.IsChecked()

    def OnCallTipsShow(self, event):
        self.shell.autoCallTip = event.IsChecked()

    def OnWrap(self, event):
        self.shell.SetWrapMode(event.IsChecked())

    def OnUpdateMenu(self, event):
        """Update menu items based on current status."""
        id = event.GetId()
        if id == wx.wxID_UNDO:
            event.Enable(self.shell.CanUndo())
        elif id == wx.wxID_REDO:
            event.Enable(self.shell.CanRedo())
        elif id == wx.wxID_CUT:
            event.Enable(self.shell.CanCut())
        elif id == wx.wxID_COPY:
            event.Enable(self.shell.CanCopy())
        elif id == ID_COPY_PLUS:
            event.Enable(self.shell.CanCopy())
        elif id == wx.wxID_PASTE:
            event.Enable(self.shell.CanPaste())
        elif id == ID_PASTE_PLUS:
            event.Enable(self.shell.CanPaste())
        elif id == wx.wxID_CLEAR:
            event.Enable(self.shell.CanCut())
        elif id == ID_AUTOCOMP_SHOW:
            event.Check(self.shell.autoComplete)
        elif id == ID_AUTOCOMP_INCLUDE_MAGIC:
            event.Check(self.shell.autoCompleteIncludeMagic)
        elif id == ID_AUTOCOMP_INCLUDE_SINGLE:
            event.Check(self.shell.autoCompleteIncludeSingle)
        elif id == ID_AUTOCOMP_INCLUDE_DOUBLE:
            event.Check(self.shell.autoCompleteIncludeDouble)
        elif id == ID_CALLTIPS_SHOW:
            event.Check(self.shell.autoCallTip)
        elif id == ID_WRAP:
            event.Check(self.shell.GetWrapMode())




# This file was created automatically by SWIG.
import mdic

from misc import *

from windows import *

from gdi import *

from fonts import *

from clip_dnd import *

from frames import *

from stattool import *

from controls import *

from events import *
import wx
class wxMDIParentFramePtr(wxFramePtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def Create(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIParentFrame_Create,(self,) + _args, _kwargs)
        return val
    def ActivateNext(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIParentFrame_ActivateNext,(self,) + _args, _kwargs)
        return val
    def ActivatePrevious(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIParentFrame_ActivatePrevious,(self,) + _args, _kwargs)
        return val
    def ArrangeIcons(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIParentFrame_ArrangeIcons,(self,) + _args, _kwargs)
        return val
    def Cascade(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIParentFrame_Cascade,(self,) + _args, _kwargs)
        return val
    def GetActiveChild(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIParentFrame_GetActiveChild,(self,) + _args, _kwargs)
        return val
    def GetClientWindow(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIParentFrame_GetClientWindow,(self,) + _args, _kwargs)
        return val
    def GetToolBar(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIParentFrame_GetToolBar,(self,) + _args, _kwargs)
        return val
    def Tile(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIParentFrame_Tile,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxMDIParentFrame instance at %s>" % (self.this,)
class wxMDIParentFrame(wxMDIParentFramePtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(mdic.new_wxMDIParentFrame,_args,_kwargs)
        self.thisown = 1
        self._setOORInfo(self)



def wxPreMDIParentFrame(*_args,**_kwargs):
    val = wxMDIParentFramePtr(apply(mdic.new_wxPreMDIParentFrame,_args,_kwargs))
    val.thisown = 1
    val._setOORInfo(val)
    return val


class wxMDIChildFramePtr(wxFramePtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def Create(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIChildFrame_Create,(self,) + _args, _kwargs)
        return val
    def Activate(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIChildFrame_Activate,(self,) + _args, _kwargs)
        return val
    def Maximize(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIChildFrame_Maximize,(self,) + _args, _kwargs)
        return val
    def Restore(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIChildFrame_Restore,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxMDIChildFrame instance at %s>" % (self.this,)
class wxMDIChildFrame(wxMDIChildFramePtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(mdic.new_wxMDIChildFrame,_args,_kwargs)
        self.thisown = 1
        self._setOORInfo(self)



def wxPreMDIChildFrame(*_args,**_kwargs):
    val = wxMDIChildFramePtr(apply(mdic.new_wxPreMDIChildFrame,_args,_kwargs))
    val.thisown = 1
    val._setOORInfo(val)
    return val


class wxMDIClientWindowPtr(wxWindowPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def Create(self, *_args, **_kwargs):
        val = apply(mdic.wxMDIClientWindow_Create,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxMDIClientWindow instance at %s>" % (self.this,)
class wxMDIClientWindow(wxMDIClientWindowPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(mdic.new_wxMDIClientWindow,_args,_kwargs)
        self.thisown = 1
        self._setOORInfo(self)



def wxPreMDIClientWindow(*_args,**_kwargs):
    val = wxMDIClientWindowPtr(apply(mdic.new_wxPreMDIClientWindow,_args,_kwargs))
    val.thisown = 1
    val._setOORInfo(val)
    return val




#-------------- FUNCTION WRAPPERS ------------------



#-------------- VARIABLE WRAPPERS ------------------

IDM_WINDOWTILE = mdic.IDM_WINDOWTILE
IDM_WINDOWTILEHOR = mdic.IDM_WINDOWTILEHOR
IDM_WINDOWCASCADE = mdic.IDM_WINDOWCASCADE
IDM_WINDOWICONS = mdic.IDM_WINDOWICONS
IDM_WINDOWNEXT = mdic.IDM_WINDOWNEXT
IDM_WINDOWTILEVERT = mdic.IDM_WINDOWTILEVERT
wxFIRST_MDI_CHILD = mdic.wxFIRST_MDI_CHILD
wxLAST_MDI_CHILD = mdic.wxLAST_MDI_CHILD



# Name:         xxx.py ('xxx' is easy to distinguish from 'wx' :) )
# Purpose:      XML interface classes
# Author:       Roman Rolinsky <rolinsky@mema.ucl.ac.be>
# Created:      22.08.2001
# RCS-ID:       $Id: xxx.py,v 1.1.2.9 2003/03/21 22:35:49 RD Exp $

from xml.dom import minidom
from globals import *
from params import *

currentEncoding = wxLocale_GetSystemEncodingName()
if not currentEncoding:
    currentEncoding = 'ascii'

# Base class for interface parameter classes
class xxxNode:
    def __init__(self, node):
        self.node = node
    def remove(self):
        self.node.parentNode.removeChild(self.node)
        self.node.unlink()

# Generic (text) parameter class
class xxxParam(xxxNode):
    # Standard use: for text nodes
    def __init__(self, node):
        xxxNode.__init__(self, node)
        if not node.hasChildNodes():
            # If does not have child nodes, create empty text node
            text = g.tree.dom.createTextNode('')
            node.appendChild(text)
        else:
            text = node.childNodes[0] # first child must be text node
            assert text.nodeType == minidom.Node.TEXT_NODE
        # Use convertion from unicode to current encoding
        self.textNode = text
    # Value returns string
    if wxUSE_UNICODE:   # no conversion is needed
        def value(self):
            return self.textNode.data
        def update(self, value):
            self.textNode.data = value
    else:
        def value(self):
            return self.textNode.data.encode(currentEncoding)
        def update(self, value):
            self.textNode.data = unicode(value, currentEncoding)

# Integer parameter
class xxxParamInt(xxxParam):
    # Standard use: for text nodes
    def __init__(self, node):
        xxxParam.__init__(self, node)
    # Value returns string
    def value(self):
        try:
            return int(self.textNode.data)
        except ValueError:
            return -1                   # invalid value
    def update(self, value):
        self.textNode.data = str(value)

# Content parameter
class xxxParamContent(xxxNode):
    def __init__(self, node):
        xxxNode.__init__(self, node)
        data, l = [], []                # data is needed to quicker value retrieval
        nodes = node.childNodes[:]      # make a copy of the child list
        for n in nodes:
            if n.nodeType == minidom.Node.ELEMENT_NODE:
                assert n.tagName == 'item', 'bad content content'
                if not n.hasChildNodes():
                    # If does not have child nodes, create empty text node
                    text = g.tree.dom.createTextNode('')
                    node.appendChild(text)
                else:
                    # !!! normalize?
                    text = n.childNodes[0] # first child must be text node
                    assert text.nodeType == minidom.Node.TEXT_NODE
                l.append(text)
                data.append(str(text.data))
            else:                       # remove other
                node.removeChild(n)
                n.unlink()
        self.l, self.data = l, data
    def value(self):
        return self.data
    def update(self, value):
        # If number if items is not the same, recreate children
        if len(value) != len(self.l):   # remove first if number of items has changed
            childNodes = self.node.childNodes[:]
            for n in childNodes:
                self.node.removeChild(n)
            l = []
            for str in value:
                itemElem = g.tree.dom.createElement('item')
                itemText = g.tree.dom.createTextNode(str)
                itemElem.appendChild(itemText)
                self.node.appendChild(itemElem)
                l.append(itemText)
            self.l = l
        else:
            for i in range(len(value)):
                self.l[i].data = value[i]
        self.data = value

# Content parameter for checklist
class xxxParamContentCheckList(xxxNode):
    def __init__(self, node):
        xxxNode.__init__(self, node)
        data, l = [], []                # data is needed to quicker value retrieval
        nodes = node.childNodes[:]      # make a copy of the child list
        for n in nodes:
            if n.nodeType == minidom.Node.ELEMENT_NODE:
                assert n.tagName == 'item', 'bad content content'
                checked = n.getAttribute('checked')
                if not checked: checked = 0
                if not n.hasChildNodes():
                    # If does not have child nodes, create empty text node
                    text = g.tree.dom.createTextNode('')
                    node.appendChild(text)
                else:
                    # !!! normalize?
                    text = n.childNodes[0] # first child must be text node
                    assert text.nodeType == minidom.Node.TEXT_NODE
                l.append((text, n))
                data.append((str(text.data), int(checked)))
            else:                       # remove other
                node.removeChild(n)
                n.unlink()
        self.l, self.data = l, data
    def value(self):
        return self.data
    def update(self, value):
        # If number if items is not the same, recreate children
        if len(value) != len(self.l):   # remove first if number of items has changed
            childNodes = self.node.childNodes[:]
            for n in childNodes:
                self.node.removeChild(n)
            l = []
            for s,ch in value:
                itemElem = g.tree.dom.createElement('item')
                # Add checked only if True
                if ch: itemElem.setAttribute('checked', '1')
                itemText = g.tree.dom.createTextNode(s)
                itemElem.appendChild(itemText)
                self.node.appendChild(itemElem)
                l.append((itemText, itemElem))
            self.l = l
        else:
            for i in range(len(value)):
                self.l[i][0].data = value[i][0]
                self.l[i][1].setAttribute('checked', str(value[i][1]))
        self.data = value

# Bitmap parameter
class xxxParamBitmap(xxxParam):
    def __init__(self, node):
        xxxParam.__init__(self, node)
        self.stock_id = node.getAttribute('stock_id')
    def value(self):
        return [self.stock_id, xxxParam.value(self)]
    def update(self, value):
        self.stock_id = value[0]
        if self.stock_id:
            self.node.setAttribute('stock_id', self.stock_id)
        else:
            self.node.removeAttribute('stock_id')
        xxxParam.update(self, value[1])

################################################################################

# Classes to interface DOM objects
class xxxObject:
    # Default behavior
    hasChildren = False                 # has children elements?
    hasStyle = True                     # almost everyone
    hasName = True                      # has name attribute?
    isSizer = hasChild = False
    allParams = None                    # Some nodes have no parameters
    # Style parameters (all optional)
    styles = ['fg', 'bg', 'font', 'enabled', 'focused', 'hidden', 'tooltip']
    # Special parameters
    specials = []
    # Bitmap tags
    bitmapTags = ['bitmap', 'bitmap2', 'icon']
    # Required paremeters: none by default
    required = []
    # Default parameters with default values
    default = {}
    # Parameter types
    paramDict = {}
    # Window styles and extended styles
    winStyles = []
    # Tree icon index
    #image = -1
    # Construct a new xxx object from DOM element
    # parent is parent xxx object (or None if none), element is DOM element object
    def __init__(self, parent, element):
        self.parent = parent
        self.element = element
        self.undo = None
        # Get attributes
        self.className = element.getAttribute('class')
        if self.hasName: self.name = element.getAttribute('name')
        # Set parameters (text element children)
        self.params = {}
        nodes = element.childNodes[:]
        for node in nodes:
            if node.nodeType == minidom.Node.ELEMENT_NODE:
                tag = node.tagName
                if tag == 'object':
                    continue            # do nothing for object children here
                if tag not in self.allParams and tag not in self.styles:
                    print 'WARNING: unknown parameter for %s: %s' % \
                          (self.className, tag)
                elif tag in self.specials:
                    self.special(tag, node)
                elif tag == 'content':
                    if self.className == 'wxCheckList':
                        self.params[tag] = xxxParamContentCheckList(node)
                    else:
                        self.params[tag] = xxxParamContent(node)
                elif tag == 'font': # has children
                    self.params[tag] = xxxParamFont(element, node)
                elif tag in self.bitmapTags:
                    # Can have attributes
                    self.params[tag] = xxxParamBitmap(node)
                else:                   # simple parameter
                    self.params[tag] = xxxParam(node)
            else:
                # Remove all other nodes
                element.removeChild(node)
                node.unlink()
        # Check that all required params are set
        for param in self.required:
            if not self.params.has_key(param):
                # If default is specified, set it
                if self.default.has_key(param):
                    elem = g.tree.dom.createElement(param)
                    self.params[param] = xxxParam(elem)
                    # Find place to put new element: first present element after param
                    found = False
                    paramStyles = self.allParams + self.styles
                    for p in paramStyles[paramStyles.index(param) + 1:]:
                        # Content params don't have same type
                        if self.params.has_key(p) and p != 'content':
                            found = True
                            break
                    if found:
                        nextTextElem = self.params[p].node
                        self.element.insertBefore(elem, nextTextElem)
                    else:
                        self.element.appendChild(elem)
                else:
                    wxLogWarning('Required parameter %s of %s missing' %
                                 (param, self.className))
    # Returns real tree object
    def treeObject(self):
        if self.hasChild: return self.child
        return self
    # Returns tree image index
    def treeImage(self):
        if self.hasChild: return self.child.treeImage()
        return self.image
    # Class name plus wx name
    def treeName(self):
        if self.hasChild: return self.child.treeName()
        if self.hasName and self.name: return self.className + ' "' + self.name + '"'
        return self.className

################################################################################

# This is a little special: it is both xxxObject and xxxNode
class xxxParamFont(xxxObject, xxxNode):
    allParams = ['size', 'style', 'weight', 'family', 'underlined',
                 'face', 'encoding']
    def __init__(self, parent, element):
        xxxObject.__init__(self, parent, element)
        xxxNode.__init__(self, element)
        self.parentNode = parent       # required to behave similar to DOM node
        v = []
        for p in self.allParams:
            try:
                v.append(str(self.params[p].value()))
            except KeyError:
                v.append('')
        self.data = v
    def update(self, value):
        # `value' is a list of strings corresponding to all parameters
        elem = self.element
        # Remove old elements first
        childNodes = elem.childNodes[:]
        for node in childNodes: elem.removeChild(node)
        i = 0
        self.params.clear()
        v = []
        for param in self.allParams:
            if value[i]:
                fontElem = g.tree.dom.createElement(param)
                textNode = g.tree.dom.createTextNode(value[i])
                self.params[param] = textNode
                fontElem.appendChild(textNode)
                elem.appendChild(fontElem)
            v.append(value[i])
            i += 1
        self.data = v
    def value(self):
        return self.data

################################################################################

class xxxContainer(xxxObject):
    hasChildren = True

# Simulate normal parameter for encoding
class xxxEncoding:
    def __init__(self, val):
        self.encd = val
    def value(self):
        return self.encd
    def update(self, val):
        self.encd = val

# Special class for root node
class xxxMainNode(xxxContainer):
    allParams = ['encoding']
    required = ['encoding']
    default = {'encoding': ''}
    hasStyle = hasName = False
    def __init__(self, dom):
        xxxContainer.__init__(self, None, dom.documentElement)
        self.className = 'XML tree'
        self.params['encoding'] = xxxEncoding(dom.encoding)

################################################################################
# Top-level windwows

class xxxPanel(xxxContainer):
    allParams = ['pos', 'size', 'style']
    styles = ['fg', 'bg', 'font', 'enabled', 'focused', 'hidden', 'exstyle',
              'tooltip']
    winStyles = ['wxNO_3D', 'wxTAB_TRAVERSAL', 'wxCLIP_CHILDREN']
    exStyles = ['wxWS_EX_VALIDATE_RECURSIVELY']

class xxxDialog(xxxContainer):
    allParams = ['title', 'centered', 'pos', 'size', 'style']
    paramDict = {'centered': ParamBool}
    required = ['title']
    winStyles = ['wxDEFAULT_DIALOG_STYLE', 'wxSTAY_ON_TOP',
                 'wxDIALOG_MODAL', 'wxDIALOG_MODELESS',
                 'wxCAPTION', 'wxSYSTEM_MENU', 'wxRESIZE_BORDER', 'wxRESIZE_BOX',
                 'wxTHICK_FRAME',
                 'wxNO_3D', 'wxTAB_TRAVERSAL', 'wxCLIP_CHILDREN']
    styles = ['fg', 'bg', 'font', 'enabled', 'focused', 'hidden', 'exstyle',
              'tooltip']
    exStyles = ['wxWS_EX_VALIDATE_RECURSIVELY']

class xxxFrame(xxxContainer):
    allParams = ['title', 'centered', 'pos', 'size', 'style']
    paramDict = {'centered': ParamBool}
    required = ['title']
    winStyles = ['wxDEFAULT_FRAME_STYLE', 'wxDEFAULT_DIALOG_STYLE',
                 'wxSTAY_ON_TOP',
                 'wxCAPTION', 'wxSYSTEM_MENU', 'wxRESIZE_BORDER',
                 'wxRESIZE_BOX', 'wxMINIMIZE_BOX', 'wxMAXIMIZE_BOX',
                 'wxFRAME_FLOAT_ON_PARENT', 'wxFRAME_TOOL_WINDOW',
                 'wxNO_3D', 'wxTAB_TRAVERSAL', 'wxCLIP_CHILDREN']
    styles = ['fg', 'bg', 'font', 'enabled', 'focused', 'hidden', 'exstyle',
              'tooltip']
    exStyles = ['wxWS_EX_VALIDATE_RECURSIVELY']

class xxxTool(xxxObject):
    allParams = ['bitmap', 'bitmap2', 'toggle', 'tooltip', 'longhelp']
    required = ['bitmap']
    paramDict = {'bitmap2': ParamBitmap, 'toggle': ParamBool}
    hasStyle = False

class xxxToolBar(xxxContainer):
    allParams = ['bitmapsize', 'margins', 'packing', 'separation',
                 'pos', 'size', 'style']
    hasStyle = False
    paramDict = {'bitmapsize': ParamPosSize, 'margins': ParamPosSize,
                 'packing': ParamInt, 'separation': ParamInt,
                 'style': ParamNonGenericStyle}
    winStyles = ['wxTB_FLAT', 'wxTB_DOCKABLE', 'wxTB_VERTICAL', 'wxTB_HORIZONTAL']

################################################################################
# Bitmap, Icon

class xxxBitmap(xxxObject):
    allParams = ['bitmap']
    required = ['bitmap']

# Just like bitmap
class xxxIcon(xxxObject):
    allParams = ['icon']
    required = ['icon']

################################################################################
# Controls

class xxxStaticText(xxxObject):
    allParams = ['label', 'pos', 'size', 'style']
    required = ['label']
    default = {'label': ''}
    winStyles = ['wxALIGN_LEFT', 'wxALIGN_RIGHT', 'wxALIGN_CENTRE', 'wxST_NO_AUTORESIZE']

class xxxStaticLine(xxxObject):
    allParams = ['pos', 'size', 'style']
    winStyles = ['wxLI_HORIZONTAL', 'wxLI_VERTICAL']

class xxxStaticBitmap(xxxObject):
    allParams = ['bitmap', 'pos', 'size', 'style']
    required = ['bitmap']

class xxxTextCtrl(xxxObject):
    allParams = ['value', 'pos', 'size', 'style']
    winStyles = ['wxTE_PROCESS_ENTER', 'wxTE_PROCESS_TAB', 'wxTE_MULTILINE',
              'wxTE_PASSWORD', 'wxTE_READONLY', 'wxHSCROLL']

class xxxChoice(xxxObject):
    allParams = ['content', 'selection', 'pos', 'size', 'style']
    required = ['content']
    winStyles = ['wxCB_SORT']

class xxxSlider(xxxObject):
    allParams = ['value', 'min', 'max', 'pos', 'size', 'style',
                 'tickfreq', 'pagesize', 'linesize', 'thumb', 'tick',
                 'selmin', 'selmax']
    paramDict = {'value': ParamInt, 'tickfreq': ParamInt, 'pagesize': ParamInt,
                 'linesize': ParamInt, 'thumb': ParamInt, 'thumb': ParamInt,
                 'tick': ParamInt, 'selmin': ParamInt, 'selmax': ParamInt}
    required = ['value', 'min', 'max']
    winStyles = ['wxSL_HORIZONTAL', 'wxSL_VERTICAL', 'wxSL_AUTOTICKS', 'wxSL_LABELS',
                 'wxSL_LEFT', 'wxSL_RIGHT', 'wxSL_TOP', 'wxSL_BOTTOM',
                 'wxSL_BOTH', 'wxSL_SELRANGE']

class xxxGauge(xxxObject):
    allParams = ['range', 'pos', 'size', 'style', 'value', 'shadow', 'bezel']
    paramDict = {'range': ParamInt, 'value': ParamInt,
                 'shadow': ParamInt, 'bezel': ParamInt}
    winStyles = ['wxGA_HORIZONTAL', 'wxGA_VERTICAL', 'wxGA_PROGRESSBAR', 'wxGA_SMOOTH']

class xxxScrollBar(xxxObject):
    allParams = ['pos', 'size', 'style', 'value', 'thumbsize', 'range', 'pagesize']
    paramDict = {'value': ParamInt, 'range': ParamInt, 'thumbsize': ParamInt,
                 'pagesize': ParamInt}
    winStyles = ['wxSB_HORIZONTAL', 'wxSB_VERTICAL']

class xxxListCtrl(xxxObject):
    allParams = ['pos', 'size', 'style']
    winStyles = ['wxLC_LIST', 'wxLC_REPORT', 'wxLC_ICON', 'wxLC_SMALL_ICON',
              'wxLC_ALIGN_TOP', 'wxLC_ALIGN_LEFT', 'wxLC_AUTOARRANGE',
              'wxLC_USER_TEXT', 'wxLC_EDIT_LABELS', 'wxLC_NO_HEADER',
              'wxLC_SINGLE_SEL', 'wxLC_SORT_ASCENDING', 'wxLC_SORT_DESCENDING']

class xxxTreeCtrl(xxxObject):
    allParams = ['pos', 'size', 'style']
    winStyles = ['wxTR_HAS_BUTTONS', 'wxTR_NO_LINES', 'wxTR_LINES_AT_ROOT',
              'wxTR_EDIT_LABELS', 'wxTR_MULTIPLE']

class xxxHtmlWindow(xxxObject):
    allParams = ['pos', 'size', 'style', 'borders', 'url', 'htmlcode']
    paramDict = {'borders': ParamInt}
    winStyles = ['wxHW_SCROLLBAR_NEVER', 'wxHW_SCROLLBAR_AUTO']

class xxxCalendarCtrl(xxxObject):
    allParams = ['pos', 'size', 'style']

class xxxNotebook(xxxContainer):
    allParams = ['usenotebooksizer', 'pos', 'size', 'style']
    paramDict = {'usenotebooksizer': ParamBool}
    winStyles = ['wxNB_FIXEDWIDTH', 'wxNB_LEFT', 'wxNB_RIGHT', 'wxNB_BOTTOM']

class xxxGenericDirCtrl(xxxObject):
    allParams = ['defaultfolder', 'filter', 'defaultfilter', 'pos', 'size', 'style']
    paramDict = {'defaultfilter': ParamInt}
    winStyles = ['wxDIRCTRL_DIR_ONLY', 'wxDIRCTRL_3D_INTERNAL', 'wxDIRCTRL_SELECT_FIRST',
                 'wxDIRCTRL_SHOW_FILTERS', 'wxDIRCTRL_EDIT_LABELS']

################################################################################
# Buttons

class xxxButton(xxxObject):
    allParams = ['label', 'default', 'pos', 'size', 'style']
    paramDict = {'default': ParamBool}
    required = ['label']
    winStyles = ['wxBU_LEFT', 'wxBU_TOP', 'wxBU_RIGHT', 'wxBU_BOTTOM']

class xxxBitmapButton(xxxObject):
    allParams = ['bitmap', 'selected', 'focus', 'disabled', 'default',
                 'pos', 'size', 'style']
    required = ['bitmap']
    winStyles = ['wxBU_AUTODRAW', 'wxBU_LEFT', 'wxBU_TOP',
                 'wxBU_RIGHT', 'wxBU_BOTTOM']

class xxxRadioButton(xxxObject):
    allParams = ['label', 'value', 'pos', 'size', 'style']
    paramDict = {'value': ParamBool}
    required = ['label']
    winStyles = ['wxRB_GROUP']

class xxxSpinButton(xxxObject):
    allParams = ['value', 'min', 'max', 'pos', 'size', 'style']
    paramDict = {'value': ParamInt}
    winStyles = ['wxSP_HORIZONTAL', 'wxSP_VERTICAL', 'wxSP_ARROW_KEYS', 'wxSP_WRAP']

class xxxSpinCtrl(xxxObject):
    allParams = ['value', 'min', 'max', 'pos', 'size', 'style']
    paramDict = {'value': ParamInt}
    winStyles = ['wxSP_HORIZONTAL', 'wxSP_VERTICAL', 'wxSP_ARROW_KEYS', 'wxSP_WRAP']

################################################################################
# Boxes

class xxxStaticBox(xxxObject):
    allParams = ['label', 'pos', 'size', 'style']
    required = ['label']

class xxxRadioBox(xxxObject):
    allParams = ['label', 'content', 'selection', 'dimension', 'pos', 'size', 'style']
    paramDict = {'dimension': ParamInt}
    required = ['label', 'content']
    winStyles = ['wxRA_SPECIFY_ROWS', 'wxRA_SPECIFY_COLS']

class xxxCheckBox(xxxObject):
    allParams = ['label', 'checked', 'pos', 'size', 'style']
    paramDict = {'checked': ParamBool}
    required = ['label']

class xxxComboBox(xxxObject):
    allParams = ['content', 'selection', 'value', 'pos', 'size', 'style']
    required = ['content']
    winStyles = ['wxCB_SIMPLE', 'wxCB_SORT', 'wxCB_READONLY', 'wxCB_DROPDOWN']

class xxxListBox(xxxObject):
    allParams = ['content', 'selection', 'pos', 'size', 'style']
    required = ['content']
    winStyles = ['wxLB_SINGLE', 'wxLB_MULTIPLE', 'wxLB_EXTENDED', 'wxLB_HSCROLL',
              'wxLB_ALWAYS_SB', 'wxLB_NEEDED_SB', 'wxLB_SORT']

class xxxCheckList(xxxObject):
    allParams = ['content', 'pos', 'size', 'style']
    required = ['content']
    winStyles = ['wxLC_LIST', 'wxLC_REPORT', 'wxLC_ICON', 'wxLC_SMALL_ICON',
              'wxLC_ALIGN_TOP', 'wxLC_ALIGN_LEFT', 'wxLC_AUTOARRANGE',
              'wxLC_USER_TEXT', 'wxLC_EDIT_LABELS', 'wxLC_NO_HEADER',
              'wxLC_SINGLE_SEL', 'wxLC_SORT_ASCENDING', 'wxLC_SORT_DESCENDING']
    paramDict = {'content': ParamContentCheckList}

################################################################################
# Sizers

class xxxSizer(xxxContainer):
    hasName = hasStyle = False
    paramDict = {'orient': ParamOrient}
    isSizer = True

class xxxBoxSizer(xxxSizer):
    allParams = ['orient']
    required = ['orient']
    default = {'orient': 'wxVERTICAL'}
    # Tree icon depends on orientation
    def treeImage(self):
        if self.params['orient'].value() == 'wxHORIZONTAL': return self.imageH
        else: return self.imageV

class xxxStaticBoxSizer(xxxBoxSizer):
    allParams = ['label', 'orient']
    required = ['label', 'orient']

class xxxGridSizer(xxxSizer):
    allParams = ['cols', 'rows', 'vgap', 'hgap']
    required = ['cols']
    default = {'cols': '2', 'rows': '2'}

# For repeated parameters
class xxxParamMulti:
    def __init__(self, node):
        self.node = node
        self.l, self.data = [], []
    def append(self, param):
        self.l.append(param)
        self.data.append(param.value())
    def value(self):
        return self.data
    def remove(self):
        for param in self.l:
            param.remove()
        self.l, self.data = [], []

class xxxFlexGridSizer(xxxGridSizer):
    specials = ['growablecols', 'growablerows']
    allParams = ['cols', 'rows', 'vgap', 'hgap'] + specials
    paramDict = {'growablecols':ParamIntList, 'growablerows':ParamIntList}
    # Special processing for growable* parameters
    # (they are represented by several nodes)
    def special(self, tag, node):
        if not self.params.has_key(tag):
            # Create new multi-group
            self.params[tag] = xxxParamMulti(node)
        self.params[tag].append(xxxParamInt(node))
    def setSpecial(self, param, value):
        # Straightforward implementation: remove, add again
        self.params[param].remove()
        del self.params[param]
        for i in value:
            node = g.tree.dom.createElement(param)
            text = g.tree.dom.createTextNode(str(i))
            node.appendChild(text)
            self.element.appendChild(node)
            self.special(param, node)

# Container with only one child.
# Not shown in tree.
class xxxChildContainer(xxxObject):
    hasName = hasStyle = False
    hasChild = True
    def __init__(self, parent, element):
        xxxObject.__init__(self, parent, element)
        # Must have one child with 'object' tag, but we don't check it
        nodes = element.childNodes[:]   # create copy
        for node in nodes:
            if node.nodeType == minidom.Node.ELEMENT_NODE:
                if node.tagName == 'object':
                    # Create new xxx object for child node
                    self.child = MakeXXXFromDOM(self, node)
                    self.child.parent = parent
                    # Copy hasChildren and isSizer attributes
                    self.hasChildren = self.child.hasChildren
                    self.isSizer = self.child.isSizer
                    return              # success
            else:
                element.removeChild(node)
                node.unlink()
        assert 0, 'no child found'

class xxxSizerItem(xxxChildContainer):
    allParams = ['option', 'flag', 'border', 'minsize']
    paramDict = {'option': ParamInt, 'minsize': ParamPosSize}
    def __init__(self, parent, element):
        xxxChildContainer.__init__(self, parent, element)
        # Remove pos parameter - not needed for sizeritems
        if 'pos' in self.child.allParams:
            self.child.allParams = self.child.allParams[:]
            self.child.allParams.remove('pos')

class xxxNotebookPage(xxxChildContainer):
    allParams = ['label', 'selected']
    paramDict = {'selected': ParamBool}
    required = ['label']
    def __init__(self, parent, element):
        xxxChildContainer.__init__(self, parent, element)
        # pos and size dont matter for notebookpages
        if 'pos' in self.child.allParams:
            self.child.allParams = self.child.allParams[:]
            self.child.allParams.remove('pos')
        if 'size' in self.child.allParams:
            self.child.allParams = self.child.allParams[:]
            self.child.allParams.remove('size')

class xxxSpacer(xxxObject):
    hasName = hasStyle = False
    allParams = ['size', 'option', 'flag', 'border']
    paramDict = {'option': ParamInt}
    default = {'size': '0,0'}

class xxxMenuBar(xxxContainer):
    allParams = ['style']
    paramDict = {'style': ParamNonGenericStyle}    # no generic styles
    winStyles = ['wxMB_DOCKABLE']

class xxxMenu(xxxContainer):
    allParams = ['label', 'help', 'style']
    default = {'label': ''}
    paramDict = {'style': ParamNonGenericStyle}    # no generic styles
    winStyles = ['wxMENU_TEAROFF']

class xxxMenuItem(xxxObject):
    allParams = ['label', 'bitmap', 'accel', 'help',
                 'checkable', 'radio', 'enabled', 'checked']
    default = {'label': ''}
    hasStyle = False

class xxxSeparator(xxxObject):
    hasName = hasStyle = False

################################################################################
# Unknown control

class xxxUnknown(xxxObject):
    allParams = ['pos', 'size', 'style']
    paramDict = {'style': ParamNonGenericStyle}    # no generic styles

################################################################################

xxxDict = {
    'wxPanel': xxxPanel,
    'wxDialog': xxxDialog,
    'wxFrame': xxxFrame,
    'tool': xxxTool,
    'wxToolBar': xxxToolBar,

    'wxBitmap': xxxBitmap,
    'wxIcon': xxxIcon,

    'wxButton': xxxButton,
    'wxBitmapButton': xxxBitmapButton,
    'wxRadioButton': xxxRadioButton,
    'wxSpinButton': xxxSpinButton,

    'wxStaticBox': xxxStaticBox,
    'wxStaticBitmap': xxxStaticBitmap,
    'wxRadioBox': xxxRadioBox,
    'wxComboBox': xxxComboBox,
    'wxCheckBox': xxxCheckBox,
    'wxListBox': xxxListBox,

    'wxStaticText': xxxStaticText,
    'wxStaticLine': xxxStaticLine,
    'wxTextCtrl': xxxTextCtrl,
    'wxChoice': xxxChoice,
    'wxSlider': xxxSlider,
    'wxGauge': xxxGauge,
    'wxScrollBar': xxxScrollBar,
    'wxTreeCtrl': xxxTreeCtrl,
    'wxListCtrl': xxxListCtrl,
    'wxCheckList': xxxCheckList,
    'wxNotebook': xxxNotebook,
    'notebookpage': xxxNotebookPage,
    'wxHtmlWindow': xxxHtmlWindow,
    'wxCalendarCtrl': xxxCalendarCtrl,
    'wxGenericDirCtrl': xxxGenericDirCtrl,
    'wxSpinCtrl': xxxSpinCtrl,

    'wxBoxSizer': xxxBoxSizer,
    'wxStaticBoxSizer': xxxStaticBoxSizer,
    'wxGridSizer': xxxGridSizer,
    'wxFlexGridSizer': xxxFlexGridSizer,
    'sizeritem': xxxSizerItem,
    'spacer': xxxSpacer,

    'wxMenuBar': xxxMenuBar,
    'wxMenu': xxxMenu,
    'wxMenuItem': xxxMenuItem,
    'separator': xxxSeparator,

    'unknown': xxxUnknown,
    }

# Create IDs for all parameters of all classes
paramIDs = {'fg': wxNewId(), 'bg': wxNewId(), 'exstyle': wxNewId(), 'font': wxNewId(),
            'enabled': wxNewId(), 'focused': wxNewId(), 'hidden': wxNewId(),
            'tooltip': wxNewId(), 'encoding': wxNewId()
            }
for cl in xxxDict.values():
    if cl.allParams:
        for param in cl.allParams + cl.paramDict.keys():
            if not paramIDs.has_key(param):
                paramIDs[param] = wxNewId()

################################################################################
# Helper functions

# Test for object elements
def IsObject(node):
    return node.nodeType == minidom.Node.ELEMENT_NODE and node.tagName == 'object'

# Make XXX object from some DOM object, selecting correct class
def MakeXXXFromDOM(parent, element):
    try:
        klass = xxxDict[element.getAttribute('class')]
    except KeyError:
        # Verify that it's not recursive exception
        print 'ERROR: unknown class:', element.getAttribute('class')
        raise
    return klass(parent, element)

# Make empty DOM element
def MakeEmptyDOM(className):
    elem = g.tree.dom.createElement('object')
    elem.setAttribute('class', className)
    # Set required and default parameters
    xxxClass = xxxDict[className]
    defaultNotRequired = filter(lambda x, l=xxxClass.required: x not in l,
                                xxxClass.default.keys())
    for param in xxxClass.required + defaultNotRequired:
        textElem = g.tree.dom.createElement(param)
        try:
            textNode = g.tree.dom.createTextNode(xxxClass.default[param])
        except KeyError:
            textNode = g.tree.dom.createTextNode('')
        textElem.appendChild(textNode)
        elem.appendChild(textElem)
    return elem

# Make empty XXX object
def MakeEmptyXXX(parent, className):
    # Make corresponding DOM object first
    elem = MakeEmptyDOM(className)
    # If parent is a sizer, we should create sizeritem object, except for spacers
    if parent:
        if parent.isSizer and className != 'spacer':
            sizerItemElem = MakeEmptyDOM('sizeritem')
            sizerItemElem.appendChild(elem)
            elem = sizerItemElem
        elif isinstance(parent, xxxNotebook):
            pageElem = MakeEmptyDOM('notebookpage')
            pageElem.appendChild(elem)
            elem = pageElem
    # Now just make object
    return MakeXXXFromDOM(parent, elem)

